@@ -1,7 +1,7 @@
 /*
  *  WINGs server.c: example how to create a network server using WMConnection
  *
- *  Copyright (c) 2001 Dan Pascu
+ *  Copyright (c) 2001-2002 Dan Pascu
  *
  */
 
@@ -52,14 +52,14 @@ static ConnectionDelegate socketDelegate = {
 
 
 void
-wAbort(Bool foo) /*FOLD00*/
+wAbort(Bool foo)
 {
     exit(1);
 }
 
 
 static void
-printHelp(char *progname) /*FOLD00*/
+printHelp(char *progname)
 {
     printf(_("usage: %s [options]\n\n"), progname);
     puts(_(" --help				print this message"));
@@ -226,7 +226,7 @@ getAvailableMessages(WMConnection *cPtr)
 
 
 static void
-complainAboutBadArgs(WMConnection *cPtr, char *cmdName, char *badArgs) /*FOLD00*/
+complainAboutBadArgs(WMConnection *cPtr, char *cmdName, char *badArgs)
 {
     char *buf = wmalloc(strlen(cmdName) + strlen(badArgs) + 100);
 
@@ -238,7 +238,7 @@ complainAboutBadArgs(WMConnection *cPtr, char *cmdName, char *badArgs) /*FOLD00*
 
 
 static void
-sendUpdateMessage(WMConnection *cPtr, char *id, int time) /*FOLD00*/
+sendUpdateMessage(WMConnection *cPtr, char *id, int time)
 {
     char *buf = wmalloc(strlen(id) + 100);
 
@@ -256,7 +256,7 @@ showId(WMConnection *cPtr)
 
 
 static void
-showHelp(WMConnection *cPtr) /*FOLD00*/
+showHelp(WMConnection *cPtr)
 {
     char *buf = wmalloc(strlen(WMGetApplicationName()) + 16);
 
@@ -290,24 +290,24 @@ showHelp(WMConnection *cPtr) /*FOLD00*/
 static void
 listUsers(WMConnection *cPtr)
 {
-    proplist_t userList;
+    WMPropList *userList;
     char *id;
     int i, time;
 
-    userList = WMGetUDAllKeys(timeDB);
+    userList = WMGetUDKeys(timeDB);
 
-    for (i=0; i<PLGetNumberOfElements(userList); i++) {
-        id = PLGetString(PLGetArrayElement(userList, i));
+    for (i=0; i<WMGetPropListItemCount(userList); i++) {
+        id = WMGetFromPLString(WMGetFromPLArray(userList, i));
         time = WMGetUDIntegerForKey(timeDB, id);
         sendUpdateMessage(cPtr, id, time);
     }
 
-    PLRelease(userList);
+    WMReleasePropList(userList);
 }
 
 
 static void
-setTimeForUser(WMConnection *cPtr, char *cmdArgs) /*FOLD00*/
+setTimeForUser(WMConnection *cPtr, char *cmdArgs)
 {
     char *id;
     int i, time;
@@ -332,7 +332,7 @@ setTimeForUser(WMConnection *cPtr, char *cmdArgs) /*FOLD00*/
 
 
 static void
-addTimeToUser(WMConnection *cPtr, char *cmdArgs) /*FOLD00*/
+addTimeToUser(WMConnection *cPtr, char *cmdArgs)
 {
     char *id;
     int i, time, newTime;
@@ -359,7 +359,7 @@ addTimeToUser(WMConnection *cPtr, char *cmdArgs) /*FOLD00*/
 
 
 static void
-subTimeFromUser(WMConnection *cPtr, char *cmdArgs) /*FOLD00*/
+subTimeFromUser(WMConnection *cPtr, char *cmdArgs)
 {
     char *id;
     int i, time, newTime;
@@ -386,7 +386,7 @@ subTimeFromUser(WMConnection *cPtr, char *cmdArgs) /*FOLD00*/
 
 
 static void
-removeTimeForUser(WMConnection *cPtr, char *cmdArgs) /*FOLD00*/
+removeTimeForUser(WMConnection *cPtr, char *cmdArgs)
 {
     char *ptr;
     int i;
@@ -412,7 +412,7 @@ removeTimeForUser(WMConnection *cPtr, char *cmdArgs) /*FOLD00*/
 
 
 static void
-getTimeForUser(WMConnection *cPtr, char *cmdArgs) /*FOLD00*/
+getTimeForUser(WMConnection *cPtr, char *cmdArgs)
 {
     char *ptr;
     int time;
@@ -525,7 +525,7 @@ isAllowedToConnect(WMConnection *cPtr)
 
 
 static void
-didReceiveInput(ConnectionDelegate *self, WMConnection *cPtr) /*FOLD00*/
+didReceiveInput(ConnectionDelegate *self, WMConnection *cPtr)
 {
     if (cPtr == serverPtr) {
         WMConnection *newPtr = WMAcceptConnection(cPtr);
@@ -548,7 +548,7 @@ didReceiveInput(ConnectionDelegate *self, WMConnection *cPtr) /*FOLD00*/
 
 
 static void
-connectionDidTimeout(ConnectionDelegate *self, WMConnection *cPtr) /*FOLD00*/
+connectionDidTimeout(ConnectionDelegate *self, WMConnection *cPtr)
 {
     WMHost *hPtr;
 
@@ -591,10 +591,16 @@ removeConnection(void *observer, WMNotification *notification)
     WMDestroyConnection(cPtr);
 }
 
+static void
+updatedDomain(void *observer, WMNotification *notification)
+{
+    wmessage("defaults domain file changed on disk. synchronizing.");
+}
+
 
 #if 0
 static Bool
-isDifferent(char *str1, char *str2) /*FOLD00*/
+isDifferent(char *str1, char *str2)
 {
     if ((!str1 && !str2) || (str1 && str2 && strcmp(str1, str2)==0))
         return False;
@@ -605,7 +611,7 @@ isDifferent(char *str1, char *str2) /*FOLD00*/
 
 
 int
-main(int argc, char **argv) /*FOLD00*/
+main(int argc, char **argv)
 {
     int i;
 
@@ -675,6 +681,8 @@ main(int argc, char **argv) /*FOLD00*/
     }
 
     timeDB = WMGetDefaultsFromPath("./UserTime.plist");
+    WMAddNotificationObserver(updatedDomain, NULL,
+                              WMUserDefaultsDidChangeNotification, NULL);
 
     clientConnections = WMCreateArray(4);
 
