@@ -106,7 +106,7 @@ void WMSetTableColumnConstraints(WMTableColumn *column,
 
 void WMSetTableColumnEditable(WMTableColumn *column, Bool flag)
 {
-    column->editable = flag;
+    column->editable = ((flag==0) ? 0 : 1);
 }
 
 
@@ -411,7 +411,8 @@ static void doScroll(WMWidget *self, void *data)
 }
 
 
-static void splitterHandler(XEvent *event, void *data)
+static void
+splitterHandler(XEvent *event, void *data)
 {
     WMTableColumn *column = (WMTableColumn*)data;
     WMTableView *table = column->table;
