@@ -1,7 +1,7 @@
 /*
  *  WINGs WMHost function library
  *
- *  Copyright (c) 1999-2001 Dan Pascu
+ *  Copyright (c) 1999-2002 Dan Pascu
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -213,7 +213,7 @@ WMReleaseHost(WMHost *hPtr)
 void
 WMSetHostCacheEnabled(Bool flag)
 {
-    hostCacheEnabled = flag;
+    hostCacheEnabled = ((flag==0) ? 0 : 1);
 }
 
 
