@@ -9,6 +9,9 @@
 #define MAX_PROPERTY_SIZE 8*1024
 
 
+char *WMSelectionOwnerDidChangeNotification = "WMSelectionOwnerDidChange";
+
+
 typedef struct SelectionHandler {
     WMView *view;
     Atom selection;
@@ -38,9 +41,14 @@ typedef struct SelectionCallback {
 } SelectionCallback;
 
 
-WMArray *selCallbacks = NULL;
 
-WMArray *selHandlers = NULL;
+static WMArray *selCallbacks = NULL;
+
+static WMArray *selHandlers = NULL;
+
+static Bool gotXError = False;
+
+
 
 
 void
@@ -53,8 +61,9 @@ WMDeleteSelectionHandler(WMView *view, Atom selection, Time timestamp)
 
     if (!selHandlers)
         return;
-    
-    
+
+    /*//printf("deleting selection handler for %d", win);*/
+
     WM_ITERATE_ARRAY(selHandlers, handler, iter) {
 	if (handler->view == view
 	    && (handler->selection == selection || selection == None)
@@ -62,13 +71,17 @@ WMDeleteSelectionHandler(WMView *view, Atom selection, Time timestamp)
 
 	    if (handler->flags.done_pending) {
 		handler->flags.delete_pending = 1;
-		return;
+                /*//puts(": postponed because still pending");*/
+                return;
 	    }
-	    WMRemoveFromArray(selHandlers, handler);
+            /*//printf(": found & removed");*/
+            WMRemoveFromArray(selHandlers, handler);
 	    break;
         }
     }
 
+    /*//printf("\n");*/
+
     XGrabServer(dpy);
     if (XGetSelectionOwner(dpy, selection) == win) {
 	XSetSelectionOwner(dpy, selection, None, timestamp);
@@ -89,7 +102,7 @@ WMDeleteSelectionCallback(WMView *view, Atom selection, Time timestamp)
     
     WM_ITERATE_ARRAY(selCallbacks, handler, iter) {
 	if (handler->view == view
-	    && (handler->selection == selection || selection == 0)
+	    && (handler->selection == selection || selection == None)
 	    && (handler->timestamp == timestamp || timestamp == CurrentTime)) {
 
 	    if (handler->flags.done_pending) {
@@ -103,45 +116,42 @@ WMDeleteSelectionCallback(WMView *view, Atom selection, Time timestamp)
 }
 
 
-
-static Bool gotError = 0;
-/*
 static int
-errorHandler(XErrorEvent *error)
+handleXError(Display *dpy, XErrorEvent *ev)
 {
-    return 0;
+    gotXError = True;
+
+    return 1;
 }
-*/
+
 
 static Bool
 writeSelection(Display *dpy, Window requestor, Atom property, Atom type,
 	       WMData *data)
 {
-    int format;
-    
+    static void *oldHandler;
+    int format, bpi;
+
     format = WMGetDataFormat(data);
     if (format == 0)
 	format = 8;
-    
-/*
-    printf("write to %x: %s\n", requestor, XGetAtomName(dpy, property));
-*/
-    gotError = False;
 
-#ifndef __sgi
-    if (!XChangeProperty(dpy, requestor, property, type, format,
-                         PropModeReplace, WMDataBytes(data),
-			 WMGetDataLength(data)))
-        return False;
-#else
-    /* in sgi seems this seems to return void */
-    XChangeProperty(dpy, requestor, property, type, format,
-		    PropModeReplace, WMDataBytes(data), WMGetDataLength(data));
-#endif
+    bpi = format/8;
+
+    /* printf("write to %x: %s\n", requestor, XGetAtomName(dpy, property)); */
+
+    oldHandler = XSetErrorHandler(handleXError);
+
+    gotXError = False;
+
+    XChangeProperty(dpy, requestor, property, type, format, PropModeReplace,
+                    WMDataBytes(data), WMGetDataLength(data)/bpi);
 
     XFlush(dpy);
 
-    return !gotError;
+    XSetErrorHandler(oldHandler);
+
+    return !gotXError;
 }
 
 
@@ -149,9 +159,9 @@ static void
 notifySelection(XEvent *event, Atom prop)
 {
     XEvent ev;
-/*
-    printf("envent to %x\n", event->xselectionrequest.requestor);
-*/
+
+    /* printf("event to %x\n", event->xselectionrequest.requestor); */
+
     ev.xselection.type = SelectionNotify;
     ev.xselection.serial = 0;
     ev.xselection.send_event = True;
@@ -174,7 +184,7 @@ handleRequestEvent(XEvent *event)
     SelectionHandler *handler;
     WMArrayIterator iter;
     WMArray *copy;
-    Bool handledRequest = False;
+    Bool handledRequest;
 
     WM_ITERATE_ARRAY(selHandlers, handler, iter) {
 
@@ -194,9 +204,8 @@ handleRequestEvent(XEvent *event)
 	    handler->flags.delete_pending = 1;
 	    break;
 
-	 case SelectionRequest:	    
-	    if (W_VIEW_DRAWABLE(handler->view)
-		!= event->xselectionrequest.owner) {
+	 case SelectionRequest:
+	    if (W_VIEW_DRAWABLE(handler->view)!=event->xselectionrequest.owner) {
 		break;
 	    }
 
@@ -213,6 +222,8 @@ handleRequestEvent(XEvent *event)
 		    break;
 		}
 
+                handledRequest = False;
+
 		handler->flags.done_pending = 1;
 
 		data = handler->procs.convertSelection(handler->view,
@@ -220,41 +231,31 @@ handleRequestEvent(XEvent *event)
 						     event->xselectionrequest.target,
 						     handler->data,
 						     &atom);
-		if (data == NULL) {
-		    break;
-		}
-	    
-		handledRequest = True;
-		
-		
+
 		prop = event->xselectionrequest.property;
 		/* obsolete clients that don't set the property field */
 		if (prop == None)
 		    prop = event->xselectionrequest.target;
-		
-		if (!writeSelection(event->xselectionrequest.display,
-				    event->xselectionrequest.requestor,
-				    prop, atom, data)) {
-		    WMReleaseData(data);
-		    notifySelection(event, None);
-		    break;
-		}
-		WMReleaseData(data);
-		
-		notifySelection(event, prop);
-		
+
+                if (data) {
+                    if (writeSelection(event->xselectionrequest.display,
+                                       event->xselectionrequest.requestor,
+                                       prop, atom, data)) {
+                        handledRequest = True;
+                    }
+                    WMReleaseData(data);
+                }
+
+                notifySelection(event, (handledRequest==True ? prop : None));
+
 		if (handler->procs.selectionDone != NULL) {
 		    handler->procs.selectionDone(handler->view,
 					 handler->selection,
 					 event->xselectionrequest.target,
 					 handler->data);
 		}
-		
+
 		handler->flags.done_pending = 0;
-		
-		if (!handledRequest) {
-		    notifySelection(event, None);
-		}
 	    }
 	    break;
 	}
@@ -278,22 +279,21 @@ getSelectionData(Display *dpy, Window win, Atom where)
     WMData *wdata;
     unsigned char *data;
     Atom rtype;
-    unsigned bits;
+    unsigned bits, bpi;
     unsigned long len, bytes;
 
-    
+
     if (XGetWindowProperty(dpy, win, where, 0, MAX_PROPERTY_SIZE, 
 			   False, AnyPropertyType, &rtype, &bits, &len,
 			   &bytes, &data)!=Success) {
 	return NULL;
     }
-    
-    wdata = WMCreateDataWithBytesNoCopy(data, len, (WMFreeDataProc*)XFree);
-    if (wdata == NULL) {
-	return NULL;
-    }
+
+    bpi = bits/8;
+
+    wdata = WMCreateDataWithBytesNoCopy(data, len*bpi, (WMFreeDataProc*)XFree);
     WMSetDataFormat(wdata, bits);
-    
+
     return wdata;
 }
 
@@ -309,7 +309,7 @@ handleNotifyEvent(XEvent *event)
     WM_ITERATE_ARRAY(selCallbacks, handler, iter) {
 	
 	if (W_VIEW_DRAWABLE(handler->view) != event->xselection.requestor
-	    && handler->selection == event->xselection.selection) {
+	    || handler->selection != event->xselection.selection) {
 	    continue;
 	}
 	handler->flags.done_pending = 1;
@@ -349,6 +349,18 @@ handleNotifyEvent(XEvent *event)
 void
 W_HandleSelectionEvent(XEvent *event)
 {
+    /*//printf("%d received selection ", event->xany.window);*/
+    /*//switch(event->type) {
+    case SelectionNotify:
+        puts("notify"); break;
+    case SelectionRequest:
+        puts("request"); break;
+    case SelectionClear:
+        puts("clear"); break;
+    default:
+        puts("unknown"); break;
+    }*/
+
     if (event->type == SelectionNotify) {
 	handleNotifyEvent(event);
     } else {
@@ -358,7 +370,6 @@ W_HandleSelectionEvent(XEvent *event)
 
 
 
-
 Bool
 WMCreateSelectionHandler(WMView *view, Atom selection, Time timestamp,
 			 WMSelectionProcs *procs, void *cdata)
@@ -367,12 +378,16 @@ WMCreateSelectionHandler(WMView *view, Atom selection, Time timestamp,
     Display *dpy = W_VIEW_SCREEN(view)->display;
 
     XSetSelectionOwner(dpy, selection, W_VIEW_DRAWABLE(view), timestamp);
-    if (XGetSelectionOwner(dpy, selection) != W_VIEW_DRAWABLE(view))
-	return False;
+    if (XGetSelectionOwner(dpy, selection) != W_VIEW_DRAWABLE(view)) {
+        return False;
+    }
 
-    handler = malloc(sizeof(SelectionHandler));
-    if (handler == NULL)
-	return False;
+    WMPostNotificationName(WMSelectionOwnerDidChangeNotification,
+                           (void*)selection, (void*)view);
+
+    /*//printf("created selection handler for %d\n", W_VIEW_DRAWABLE(view));*/
+
+    handler = wmalloc(sizeof(SelectionHandler));
 
     handler->view = view;
     handler->selection = selection;
@@ -384,9 +399,9 @@ WMCreateSelectionHandler(WMView *view, Atom selection, Time timestamp,
     if (selHandlers == NULL) {
 	selHandlers = WMCreateArrayWithDestructor(4, wfree);
     }
-    
+
     WMAddToArray(selHandlers, handler);
-    
+
     return True;
 }
 
@@ -400,9 +415,15 @@ WMRequestSelection(WMView *view, Atom selection, Atom target, Time timestamp,
 
     if (XGetSelectionOwner(W_VIEW_SCREEN(view)->display, selection) == None)
 	return False;
-    
+
+    if (!XConvertSelection(W_VIEW_SCREEN(view)->display, selection, target,
+			   W_VIEW_SCREEN(view)->clipboardAtom,
+			   W_VIEW_DRAWABLE(view), timestamp)) {
+        return False;
+    }
+
     handler = wmalloc(sizeof(SelectionCallback));
-    
+
     handler->view = view;
     handler->selection = selection;
     handler->target = target;
@@ -410,19 +431,13 @@ WMRequestSelection(WMView *view, Atom selection, Atom target, Time timestamp,
     handler->callback = callback;
     handler->data = cdata;
     memset(&handler->flags, 0, sizeof(handler->flags));
-    
+
     if (selCallbacks == NULL) {
 	selCallbacks = WMCreateArrayWithDestructor(4, wfree);
     }
 
     WMAddToArray(selCallbacks, handler);
 
-    if (!XConvertSelection(W_VIEW_SCREEN(view)->display, selection, target,
-			   W_VIEW_SCREEN(view)->clipboardAtom,
-			   W_VIEW_DRAWABLE(view), timestamp)) {
-	return False;
-    }
-    
     return True;
 }
 
