@@ -1,4 +1,6 @@
 
+#include "wconfig.h"
+
 #include <string.h>
 #include <stdlib.h>
 #include <ctype.h>
@@ -217,4 +219,17 @@ wstrappend(char *dst, char *src)
 }
 
 
+#ifndef HAVE_STRCASECMP
+int
+strcasecmp(const char *s1, const char *s2)
+{
+    while (*s1 && *s2 && (tolower(*s1)==tolower(*s2))) {
+        s1++;
+        s2++;
+    }
+
+    return (tolower(*s1) - tolower(*s2));
+}
+#endif
+
 
