@@ -44,7 +44,7 @@ generalize_xlfd (const char *xlfd)
     char *pxlsz  = xlfd_get_element(xlfd, 7);
 
 #define Xstrlen(A) ((A)?strlen(A):0)
-    len = Xstrlen(xlfd)+Xstrlen(weight)+Xstrlen(slant)+Xstrlen(pxlsz)*2+50;
+    len = Xstrlen(xlfd)+Xstrlen(weight)+Xstrlen(slant)+Xstrlen(pxlsz)*2+60;
 #undef Xstrlen
 
     buf = wmalloc(len + 1);
@@ -106,7 +106,7 @@ WMCreateFontSet(WMScreen *scrPtr, char *fontName)
     char *defaultString;
     XFontSetExtents *extents;
 
-    font = WMHashGet(scrPtr->fontCache, fontName);
+    font = WMHashGet(scrPtr->fontSetCache, fontName);
     if (font) {
 	WMRetainFont(font);
 	return font;
@@ -114,7 +114,7 @@ WMCreateFontSet(WMScreen *scrPtr, char *fontName)
 
     font = malloc(sizeof(WMFont));
     if (!font)
-      return NULL;
+        return NULL;
     memset(font, 0, sizeof(WMFont));
 
     font->notFontSet = 0;
@@ -150,7 +150,7 @@ WMCreateFontSet(WMScreen *scrPtr, char *fontName)
 
     font->name = wstrdup(fontName);
 
-    assert(WMHashInsert(scrPtr->fontCache, font->name, font)==NULL);
+    assert(WMHashInsert(scrPtr->fontSetCache, font->name, font)==NULL);
 
     return font;
 }
@@ -193,7 +193,8 @@ WMCreateNormalFont(WMScreen *scrPtr, char *fontName)
     font->font.normal = XLoadQueryFont(display, fname);
     if (!font->font.normal) {
 	wfree(font);
-	return NULL;
+        wfree(fname);
+        return NULL;
     }
 
     font->height = font->font.normal->ascent+font->font.normal->descent;
@@ -245,7 +246,11 @@ WMReleaseFont(WMFont *font)
 	    XFreeFontSet(font->screen->display, font->font.set);
 	}
 	if (font->name) {
-	    WMHashRemove(font->screen->fontCache, font->name);
+            if (font->notFontSet) {
+                WMHashRemove(font->screen->fontCache, font->name);
+            } else {
+                WMHashRemove(font->screen->fontSetCache, font->name);
+            }
 	    wfree(font->name);
 	}
 	wfree(font);
@@ -263,6 +268,19 @@ WMFontHeight(WMFont *font)
 }
 
 
+WMFont*
+WMDefaultSystemFont(WMScreen *scrPtr)
+{
+    return WMRetainFont(scrPtr->normalFont);
+}
+
+
+WMFont*
+WMDefaultBoldSystemFont(WMScreen *scrPtr)
+{
+    return WMRetainFont(scrPtr->boldFont);
+}
+
 
 WMFont*
 WMSystemFontOfSize(WMScreen *scrPtr, int size)
@@ -455,132 +473,137 @@ makeFontSetOfSize(char *fontset, int size)
 static void
 changeFontProp(char *fname, char *newprop, int which)
 {
-	char before[128], prop[128], after[128];
-	char *ptr, *bptr;
-	int part=0;
-	
-	if(!fname || !prop)
-		return;
+    char before[128], prop[128], after[128];
+    char *ptr, *bptr;
+    int part=0;
 
-	ptr = fname;
-	bptr = before;
-	while (*ptr) {
-		if(*ptr == '-') {
-    	    *bptr = 0;
-			if(part==which) bptr = prop;
-			else if(part==which+1) bptr = after;
-    	  	*bptr++ = *ptr;
-			part++;
-		} else {
-    	  *bptr++ = *ptr;
-		}  ptr++;
-	}*bptr = 0;
-	snprintf(fname, 255, "%s-%s%s", before, newprop, after); 
+    if(!fname || !prop)
+        return;
+
+    ptr = fname;
+    bptr = before;
+    while (*ptr) {
+        if(*ptr == '-') {
+            *bptr = 0;
+            if(part==which)
+                bptr = prop;
+            else if(part==which+1)
+                bptr = after;
+            *bptr++ = *ptr;
+            part++;
+        } else {
+            *bptr++ = *ptr;
+        }
+        ptr++;
+    }
+    *bptr = 0;
+    snprintf(fname, 255, "%s-%s%s", before, newprop, after);
 }
 
 
-WMFont *
+WMFont*
 WMNormalizeFont(WMScreen *scr, WMFont *font)
-{   
+{
     WMFont *newfont=NULL;
     char fname[256];
-    
+
     if(!scr || !font)
         return NULL;
-    
-    snprintf(fname, 255, font->name); 
+
+    snprintf(fname, 255, font->name);
     changeFontProp(fname, "medium", 2);
-	changeFontProp(fname, "r", 3);
+    changeFontProp(fname, "r", 3);
     newfont = WMCreateNormalFont(scr, fname);
-    
+
     if(!newfont)
         return NULL;
-    
+
     return newfont;
 }
 
 
-WMFont *
-WMStrengthenFont(WMScreen *scr, WMFont *font) 
+WMFont*
+WMStrengthenFont(WMScreen *scr, WMFont *font)
 {
-	WMFont *newfont=NULL;
-	char fname[256];
+    WMFont *newfont=NULL;
+    char fname[256];
 
-	if(!scr || !font)
+    if(!scr || !font)
         return NULL;
 
-	snprintf(fname, 255, font->name);
-	changeFontProp(fname, "bold", 2);
-	newfont = WMCreateNormalFont(scr, fname);
+    snprintf(fname, 255, font->name);
+    changeFontProp(fname, "bold", 2);
+    newfont = WMCreateNormalFont(scr, fname);
 
-	if(!newfont)
+    if(!newfont)
         return NULL;
 
-	return newfont;
+    return newfont;
 }
 
 
-WMFont *
-WMUnstrengthenFont(WMScreen *scr, WMFont *font) 
+WMFont*
+WMUnstrengthenFont(WMScreen *scr, WMFont *font)
 {
-	WMFont *newfont=NULL;
-	char fname[256];
+    WMFont *newfont=NULL;
+    char fname[256];
 
-	if(!scr || !font)
+    if(!scr || !font)
         return NULL;
 
-	snprintf(fname, 255, font->name);
-	changeFontProp(fname, "medium", 2);
-	newfont = WMCreateNormalFont(scr, fname);
+    snprintf(fname, 255, font->name);
+    changeFontProp(fname, "medium", 2);
+    newfont = WMCreateNormalFont(scr, fname);
 
-	if(!newfont)
+    if(!newfont)
         return NULL;
 
-	return newfont;
+    return newfont;
 }
 
 
-WMFont *
-WMEmphasizeFont(WMScreen *scr, WMFont *font) 
+WMFont*
+WMEmphasizeFont(WMScreen *scr, WMFont *font)
 {
-	WMFont *newfont=NULL;
-	char fname[256];
+    WMFont *newfont=NULL;
+    char fname[256];
 
-	if(!scr || !font)
+    if(!scr || !font)
         return NULL;
 
-	snprintf(fname, 255, font->name);
-	changeFontProp(fname, "o", 3);
-	newfont = WMCreateNormalFont(scr, fname);
+    snprintf(fname, 255, font->name);
+    changeFontProp(fname, "o", 3);
+    newfont = WMCreateNormalFont(scr, fname);
 
-	if(!newfont)
+    if(!newfont)
         return NULL;
 
-	return newfont;
+    return newfont;
 }
 
 
-WMFont *
-WMUnemphasizeFont(WMScreen *scr, WMFont *font) 
+WMFont*
+WMUnemphasizeFont(WMScreen *scr, WMFont *font)
 {
-	WMFont *newfont=NULL;
-	char fname[256];
+    WMFont *newfont=NULL;
+    char fname[256];
 
-	if(!scr || !font)
+    if(!scr || !font)
         return NULL;
 
-	snprintf(fname, 255, font->name);
-	changeFontProp(fname, "r", 3);
-	newfont = WMCreateNormalFont(scr, fname);
+    snprintf(fname, 255, font->name);
+    changeFontProp(fname, "r", 3);
+    newfont = WMCreateNormalFont(scr, fname);
 
-	if(!newfont)
+    if(!newfont)
         return NULL;
 
-	return newfont;
+    return newfont;
 }
 
-WMFont *
-WMGetFontOfSize(WMScreen *scr, WMFont *font, int size) 
+
+WMFont*
+WMGetFontOfSize(WMScreen *scr, WMFont *font, int size)
 {
     if(!scr || !font || size<1)
         return NULL;
