@@ -176,11 +176,14 @@ W_PaintText(W_View *view, Drawable d, WMFont *font,  int x, int y,
 
 	WMDrawString(view->screen, d, gc, font, line_x, y, ptr, count);
 
-	y += fheight;
+        if (wrap && ptr[count]!='\n')
+            y += fheight;
+
+        while (ptr[count] && ptr[count]=='\n') {
+            y += fheight;
+            count++;
+        }
 
-	if (isspace(ptr[count]))
-	    count++;
-	
 	ptr += count;
 	length -= count;
     }
@@ -231,10 +234,10 @@ W_PaintTextAndImage(W_View *view, int wrap, GC textGC, W_Font *font,
 	w = view->size.width;
 	h = view->size.height;
     } else {
-	x = 2;
-	y = 2;
-	w = view->size.width - 4;
-	h = view->size.height - 4;	
+	x = 1;
+	y = 1;
+	w = view->size.width - 3;
+	h = view->size.height - 3;	
     }
 
     /* calc. image alignment */
