@@ -115,24 +115,24 @@ static void didResizeTextField();
 
 struct W_ViewDelegate _TextFieldViewDelegate = {
     NULL,
-	NULL,
-	didResizeTextField,
-	NULL,
-	NULL
+    NULL,
+    didResizeTextField,
+    NULL,
+    NULL
 };
 
 
 
-static void lostHandler(WMView *view, Atom selection, void *cdata);
+static void lostSelection(WMView *view, Atom selection, void *cdata);
 
 static WMData *requestHandler(WMView *view, Atom selection, Atom target,
-			      void *cdata, Atom *type);
+                              void *cdata, Atom *type);
 
 
 static WMSelectionProcs selectionHandler = {
     requestHandler,
-	lostHandler,
-	NULL
+    lostSelection,
+    NULL
 };
 
 
@@ -234,7 +234,7 @@ decrToFit(TextField *tPtr)
 
 static WMData*
 requestHandler(WMView *view, Atom selection, Atom target, void *cdata, 
-	       Atom *type)
+               Atom *type)
 {
     TextField *tPtr = view->self;
     int count;
@@ -281,22 +281,34 @@ requestHandler(WMView *view, Atom selection, Atom target, void *cdata,
 
 
 static void
-lostHandler(WMView *view, Atom selection, void *cdata)
+lostSelection(WMView *view, Atom selection, void *cdata)
 {
     TextField *tPtr = (WMTextField*)view->self;
 
-    tPtr->flags.ownsSelection = 0;
-    tPtr->selection.count = 0;
-    paintTextField(tPtr);
+    if (tPtr->flags.ownsSelection) {
+        WMDeleteSelectionHandler(view, selection, CurrentTime);
+        tPtr->flags.ownsSelection = 0;
+    }
+    if (tPtr->selection.count != 0) {
+        tPtr->selection.count = 0;
+        paintTextField(tPtr);
+    }
 }
 
 
 static void
-_notification(void *observerData, WMNotification *notification)
+selectionNotification(void *observerData, WMNotification *notification)
 {
-    WMTextField *to = (WMTextField*)observerData;
-    WMTextField *tw = (WMTextField*)WMGetNotificationClientData(notification);
-    if (to != tw) lostHandler(to->view, XA_PRIMARY, NULL);
+    WMView *observerView = (WMView*)observerData;
+    WMView *newOwnerView = (WMView*)WMGetNotificationClientData(notification);
+
+    if (observerView != newOwnerView) {
+        /*
+        //if (tPtr->flags.ownsSelection)
+        //    WMDeleteSelectionHandler(observerView, XA_PRIMARY, CurrentTime);
+        */
+        lostSelection(observerView, XA_PRIMARY, NULL);
+    }
 }
 
 
@@ -347,8 +359,10 @@ WMCreateTextField(WMWidget *parent)
     WMCreateEventHandler(tPtr->view, EnterWindowMask|LeaveWindowMask
 			 |ButtonReleaseMask|ButtonPressMask|KeyPressMask|Button1MotionMask,
 			 handleTextFieldActionEvents, tPtr);
-    
-    WMAddNotificationObserver(_notification, tPtr, "_lostOwnership", tPtr);
+
+    WMAddNotificationObserver(selectionNotification, tPtr->view,
+                              WMSelectionOwnerDidChangeNotification,
+                              (void*)XA_PRIMARY);
 
 
     tPtr->flags.cursorOn = 1;
@@ -366,6 +380,15 @@ WMSetTextFieldDelegate(WMTextField *tPtr, WMTextFieldDelegate *delegate)
 }
 
 
+WMTextFieldDelegate*
+WMGetTextFieldDelegate(WMTextField *tPtr)
+{
+    CHECK_CLASS(tPtr, WC_TextField);
+
+    return tPtr->delegate;
+}
+
+
 void
 WMInsertTextFieldText(WMTextField *tPtr, char *text, int position)
 {
@@ -515,7 +538,7 @@ WMSetTextFieldBeveled(WMTextField *tPtr, Bool flag)
 {
     CHECK_CLASS(tPtr, WC_TextField);
     
-    tPtr->flags.beveled = flag;
+    tPtr->flags.beveled = ((flag==0) ? 0 : 1);
 
     if (tPtr->view->flags.realized) {
 	paintTextField(tPtr);
@@ -529,7 +552,7 @@ WMSetTextFieldSecure(WMTextField *tPtr, Bool flag)
 {
     CHECK_CLASS(tPtr, WC_TextField);
     
-    tPtr->flags.secure = flag;
+    tPtr->flags.secure = ((flag==0) ? 0 : 1);
     
     if (tPtr->view->flags.realized) {
 	paintTextField(tPtr);
@@ -551,7 +574,7 @@ WMSetTextFieldEditable(WMTextField *tPtr, Bool flag)
 {
     CHECK_CLASS(tPtr, WC_TextField);
     
-    tPtr->flags.enabled = flag;
+    tPtr->flags.enabled = ((flag==0) ? 0 : 1);
     
     if (tPtr->view->flags.realized) {
 	paintTextField(tPtr);
@@ -1001,52 +1024,49 @@ handleTextFieldKeyPress(TextField *tPtr, XEvent *event)
     int cancelSelection = 1;
     Bool shifted, controled, modified;
     Bool relay = True;
-    WMScreen *scr = tPtr->view->screen;
 
     /*printf("(%d,%d) -> ", tPtr->selection.position, tPtr->selection.count);*/
     if (((XKeyEvent *) event)->state & WM_EMACSKEYMASK)
 	control_pressed = 1;
-    
-    shifted = event->xkey.state & ShiftMask;
-    controled = event->xkey.state & ControlMask;
-    if ((event->xkey.state & (ShiftMask|ControlMask)) != 0) {
-	modified = True;
-    } else {
-	modified = False;
-    }
+
+    shifted = (event->xkey.state & ShiftMask ? True : False);
+    controled = (event->xkey.state & ControlMask ? True : False);
+    modified = shifted || controled;
 
     count = XLookupString(&event->xkey, buffer, 63, &ksym, NULL);
     buffer[count] = '\0';
-    
+
     switch (ksym) {
      case XK_Tab:
 #ifdef XK_ISO_Left_Tab
      case XK_ISO_Left_Tab:
 #endif
-	if (!controled && !modified) {
+	if (!controled) {
 	    if (shifted) {
 		if (tPtr->view->prevFocusChain) {
 		    W_SetFocusOfTopLevel(W_TopLevelOfView(tPtr->view),
 					 tPtr->view->prevFocusChain);
-		    tPtr->flags.notIllegalMovement = 1;
+                    tPtr->flags.notIllegalMovement = 1;
 		}
 		data = (void*)WMBacktabTextMovement;
 	    } else {
 		if (tPtr->view->nextFocusChain) {
 		    W_SetFocusOfTopLevel(W_TopLevelOfView(tPtr->view),
 					 tPtr->view->nextFocusChain);
-		    tPtr->flags.notIllegalMovement = 1;
+                    tPtr->flags.notIllegalMovement = 1;
 		}
 		data = (void*)WMTabTextMovement;
 	    }
 	    textEvent = WMTextDidEndEditingNotification;
 
+            cancelSelection = 0;
+
 	    relay = False;
 	}
 	break;
 
      case XK_Escape:
-	if (!shifted && !controled && !modified) {
+	if (!modified) {
 	    data = (void*)WMEscapeTextMovement;
 	    textEvent = WMTextDidEndEditingNotification;
 	    
@@ -1055,7 +1075,7 @@ handleTextFieldKeyPress(TextField *tPtr, XEvent *event)
 	break;
 
      case XK_Return:
-	if (!shifted && !controled && !modified) {
+	if (!modified) {
 	    data = (void*)WMReturnTextMovement;
 	    textEvent = WMTextDidEndEditingNotification;
 	    
@@ -1067,89 +1087,87 @@ handleTextFieldKeyPress(TextField *tPtr, XEvent *event)
 	if (!control_pressed)
 	    goto normal_key;
 	else
-	    modified = False;
+            controled = False;
+
 #ifdef XK_KP_Left
      case XK_KP_Left:
 #endif
      case XK_Left:
-	if (!modified) {
-	    if (tPtr->cursorPosition > 0) {
-		paintCursor(tPtr);
-		if (event->xkey.state & ControlMask) {
-		    int i = tPtr->cursorPosition - 1;
-		    
-		    while (i > 0 && tPtr->text[i] != ' ') i--;
-		    while (i > 0 && tPtr->text[i] == ' ') i--;
-		    
-		    tPtr->cursorPosition = (i > 0) ? i + 1 : 0;
-		} else
-		    tPtr->cursorPosition--;
-		
-		if (tPtr->cursorPosition < tPtr->viewPosition) {
-		    tPtr->viewPosition = tPtr->cursorPosition;
-		    refresh = 1;
-		} else
-		    paintCursor(tPtr);
-	    }
-	    if (event->xkey.state & ShiftMask)
-		cancelSelection = 0;
-	    
-	    relay = False;
-	}
+        if (tPtr->cursorPosition > 0) {
+            paintCursor(tPtr);
+            if (controled) {
+                int i = tPtr->cursorPosition - 1;
+
+                while (i > 0 && tPtr->text[i] != ' ') i--;
+                while (i > 0 && tPtr->text[i] == ' ') i--;
+
+                tPtr->cursorPosition = (i > 0) ? i + 1 : 0;
+            } else
+                tPtr->cursorPosition--;
+
+            if (tPtr->cursorPosition < tPtr->viewPosition) {
+                tPtr->viewPosition = tPtr->cursorPosition;
+                refresh = 1;
+            } else
+                paintCursor(tPtr);
+        }
+        if (shifted)
+            cancelSelection = 0;
+
+        relay = False;
+
 	break;
 
     case WM_EMACSKEY_RIGHT:
         if (!control_pressed)
             goto normal_key;
         else
-	    modified = False;
+	    controled = False;
 
 #ifdef XK_KP_Right
     case XK_KP_Right:
 #endif
     case XK_Right:
-	if (!modified) {
-	    if (tPtr->cursorPosition < tPtr->textLen) {
-		paintCursor(tPtr);
-		if (event->xkey.state & ControlMask) {
-		    int i = tPtr->cursorPosition;
-		    
-		    while (tPtr->text[i] && tPtr->text[i] != ' ') i++;
-		    while (tPtr->text[i] == ' ') i++;
-		    
-		    tPtr->cursorPosition = i;
-		} else {
-		    tPtr->cursorPosition++;
-		}
-		while (WMWidthOfString(tPtr->font,
-				       &(tPtr->text[tPtr->viewPosition]),
-				       tPtr->cursorPosition-tPtr->viewPosition)
-		       > tPtr->usableWidth) {
-		    tPtr->viewPosition++;
-		    refresh = 1;
-		}
-		if (!refresh)
-		    paintCursor(tPtr);
-	    }
-	    if (event->xkey.state & ShiftMask)
-		cancelSelection = 0;
-	    
-	    relay = False;
-	}
+        if (tPtr->cursorPosition < tPtr->textLen) {
+            paintCursor(tPtr);
+            if (controled) {
+                int i = tPtr->cursorPosition;
+
+                while (tPtr->text[i] && tPtr->text[i] != ' ') i++;
+                while (tPtr->text[i] == ' ') i++;
+
+                tPtr->cursorPosition = i;
+            } else {
+                tPtr->cursorPosition++;
+            }
+            while (WMWidthOfString(tPtr->font,
+                                   &(tPtr->text[tPtr->viewPosition]),
+                                   tPtr->cursorPosition-tPtr->viewPosition)
+                   > tPtr->usableWidth) {
+                tPtr->viewPosition++;
+                refresh = 1;
+            }
+            if (!refresh)
+                paintCursor(tPtr);
+        }
+        if (shifted)
+            cancelSelection = 0;
+
+        relay = False;
+
 	break;
 	
     case WM_EMACSKEY_HOME:
         if (!control_pressed)
             goto normal_key;
-        else {
-	    modified = False;
-	    controled = False;
-	}
+        else
+            controled = False;
+
 #ifdef XK_KP_Home
     case XK_KP_Home:
 #endif
     case XK_Home:
-	if (!modified && !controled) {
+	if (!controled) {
 	    if (tPtr->cursorPosition > 0) {
 		paintCursor(tPtr);
 		tPtr->cursorPosition = 0;
@@ -1159,7 +1177,7 @@ handleTextFieldKeyPress(TextField *tPtr, XEvent *event)
 		} else
 		    paintCursor(tPtr);
 	    }
-	    if (event->xkey.state & ShiftMask)
+	    if (shifted)
 		cancelSelection = 0;
 	    
 	    relay = False;
@@ -1169,15 +1187,14 @@ handleTextFieldKeyPress(TextField *tPtr, XEvent *event)
     case WM_EMACSKEY_END:
         if (!control_pressed)
             goto normal_key;
-        else {
-	    modified = False;
-	    controled = False;
-	}
+        else
+            controled = False;
+
 #ifdef XK_KP_End
     case XK_KP_End:
 #endif
     case XK_End:
-	if (!modified && !controled) {
+	if (!controled) {
 	    if (tPtr->cursorPosition < tPtr->textLen) {
 		paintCursor(tPtr);
 		tPtr->cursorPosition = tPtr->textLen;
@@ -1192,7 +1209,7 @@ handleTextFieldKeyPress(TextField *tPtr, XEvent *event)
 		if (!refresh)
 		    paintCursor(tPtr);
 	    }
-	    if (event->xkey.state & ShiftMask)
+	    if (shifted)
 		cancelSelection = 0;
 	    
 	    relay = False;
@@ -1201,14 +1218,12 @@ handleTextFieldKeyPress(TextField *tPtr, XEvent *event)
 	
      case WM_EMACSKEY_BS:
          if (!control_pressed)
-            goto normal_key;
-         else {
-	    modified = False;
-	    controled = False;
-	    shifted = False;
-	}
+             goto normal_key;
+         else
+             modified = False;
+
      case XK_BackSpace:
-	if (!modified && !shifted && !controled) {
+	if (!modified) {
 	    if (tPtr->selection.count) {
 		WMDeleteTextFieldRange(tPtr, tPtr->selection);
 		data = (void*)WMDeleteTextEvent;
@@ -1229,16 +1244,14 @@ handleTextFieldKeyPress(TextField *tPtr, XEvent *event)
     case WM_EMACSKEY_DEL:
         if (!control_pressed)
             goto normal_key;
-	else {
-	    modified = False;
-	    controled = False;
-	    shifted = False;
-        }
+	else
+            modified = False;
+
 #ifdef XK_KP_Delete
     case XK_KP_Delete:
 #endif
     case XK_Delete:
-	if (!modified && !controled && !shifted) {
+	if (!modified) {
 	    if (tPtr->selection.count) {
 		WMDeleteTextFieldRange(tPtr, tPtr->selection);
 		data = (void*)WMDeleteTextEvent;
@@ -1259,7 +1272,7 @@ handleTextFieldKeyPress(TextField *tPtr, XEvent *event)
     normal_key:
      default:
 	if (!controled) {
-	    if (count > 0 && isprint(buffer[0])) {
+	    if (count > 0 && !iscntrl(buffer[0])) {
 		if (tPtr->selection.count)
 		    WMDeleteTextFieldRange(tPtr, tPtr->selection);
 		WMInsertTextFieldText(tPtr, buffer, tPtr->cursorPosition);
@@ -1279,7 +1292,7 @@ handleTextFieldKeyPress(TextField *tPtr, XEvent *event)
 
     /* Do not allow text selection in secure text fields */
     if (cancelSelection || tPtr->flags.secure) {
-        lostHandler(tPtr->view, XA_PRIMARY, NULL);
+        lostSelection(tPtr->view, XA_PRIMARY, NULL);
 
         if (tPtr->selection.count) {
             tPtr->selection.count = 0;
@@ -1291,8 +1304,6 @@ handleTextFieldKeyPress(TextField *tPtr, XEvent *event)
 
             tPtr->selection.count = tPtr->cursorPosition - tPtr->selection.position;
 
-            WMPostNotificationName("_lostOwnership", NULL, tPtr);
-
             refresh = 1;
         }
     }
@@ -1448,16 +1459,6 @@ handleTextFieldActionEvents(XEvent *event, void *data)
 
             tPtr->selection.count = tPtr->cursorPosition - tPtr->selection.position;
 
-	    if (tPtr->selection.count != 0) {
-		if (!tPtr->flags.ownsSelection) {
-		    WMCreateSelectionHandler(tPtr->view,
-					     XA_PRIMARY,
-					     event->xbutton.time,
-					     &selectionHandler, NULL);
-		    tPtr->flags.ownsSelection = 1;
-		}
-	    }
-	    
 	    paintCursor(tPtr);
 	    paintTextField(tPtr);
 
@@ -1483,7 +1484,8 @@ handleTextFieldActionEvents(XEvent *event, void *data)
             textWidth = WMWidthOfString(tPtr->font, tPtr->text, tPtr->textLen);
             if (tPtr->flags.enabled && !tPtr->flags.focused) {
                 WMSetFocusToWidget(tPtr);
-            } else if (tPtr->flags.focused) {
+            }
+            if (tPtr->flags.focused) {
 		tPtr->selection.position = tPtr->cursorPosition;
                 tPtr->selection.count = 0;
             }
@@ -1500,11 +1502,8 @@ handleTextFieldActionEvents(XEvent *event, void *data)
          case WALeft:
             if (tPtr->flags.enabled && !tPtr->flags.focused) {
                 WMSetFocusToWidget(tPtr);
-                tPtr->cursorPosition = pointToCursorPosition(tPtr, 
-                                     event->xbutton.x);
-                paintTextField(tPtr);
-            } else if (tPtr->flags.focused 
-		       && event->xbutton.button == Button1) {
+            }
+            if (tPtr->flags.focused && event->xbutton.button == Button1) {
                 tPtr->cursorPosition = pointToCursorPosition(tPtr, 
                                      event->xbutton.x);
 		tPtr->selection.position = tPtr->cursorPosition;
@@ -1547,7 +1546,7 @@ handleTextFieldActionEvents(XEvent *event, void *data)
 	    break;
 	}
 	
-	if (tPtr->selection.count != 0) {
+	if (!tPtr->flags.secure && tPtr->selection.count!=0) {
 	    int start, count;
 	    XRotateBuffers(dpy, 1);
 	    
@@ -1570,15 +1569,21 @@ handleTextFieldActionEvents(XEvent *event, void *data)
 	    paintTextField(tPtr);
 	    
 	    if (!tPtr->flags.ownsSelection) {
-		WMCreateSelectionHandler(tPtr->view,
-					 XA_PRIMARY,
-					 event->xbutton.time,
-					 &selectionHandler, NULL);
-		tPtr->flags.ownsSelection = 1;
-	    }
-	    
-	    WMPostNotificationName("_lostOwnership", NULL, tPtr);
-	}
+                tPtr->flags.ownsSelection =
+                    WMCreateSelectionHandler(tPtr->view,
+                                             XA_PRIMARY,
+                                             event->xbutton.time,
+                                             &selectionHandler, NULL);
+            }
+	} else if (!tPtr->flags.secure && tPtr->selection.count!=0 &&
+                   !tPtr->flags.ownsSelection) {
+            tPtr->flags.ownsSelection =
+                WMCreateSelectionHandler(tPtr->view,
+                                         XA_PRIMARY,
+                                         event->xbutton.time,
+                                         &selectionHandler, NULL);
+        }
+
 	lastButtonReleasedEvent = event->xbutton.time;
 		
         break;
@@ -1595,6 +1600,7 @@ destroyTextField(TextField *tPtr)
 #endif
 
     WMReleaseFont(tPtr->font);
+    /*// use lostSelection() instead of WMDeleteSelectionHandler here?*/
     WMDeleteSelectionHandler(tPtr->view, XA_PRIMARY, CurrentTime);
     WMRemoveNotificationObserver(tPtr);
 
@@ -1603,3 +1609,5 @@ destroyTextField(TextField *tPtr)
 
     wfree(tPtr);
 }
+
+
