@@ -112,9 +112,7 @@ createView(W_Screen *screen, W_View *parent)
     
     view = wmalloc(sizeof(W_View));
     memset(view, 0, sizeof(W_View));
-    
-    view->refCount = 1;
-    
+
     view->screen = screen;
 
     if (parent!=NULL) {
@@ -132,6 +130,10 @@ createView(W_Screen *screen, W_View *parent)
 	adoptChildView(parent, view);
     }
 
+    view->refCount = 1;
+
+    view->eventHandlers = WMCreateArrayWithDestructor(4, wfree);
+
     return view;
 }
 
@@ -182,6 +184,24 @@ W_CreateTopView(W_Screen *screen)
 }
 
 
+W_View*
+W_CreateUnmanagedTopView(W_Screen *screen)
+{
+    W_View *view;
+    
+    view = createView(screen, screen->rootView);
+    if (!view)
+	return NULL;
+
+    view->flags.topLevel = 1;
+    view->attribs.event_mask |= StructureNotifyMask;
+
+    view->attribFlags |= CWOverrideRedirect;
+    view->attribs.override_redirect = True;
+
+    return view;
+}
+
 
 void
 W_RealizeView(W_View *view)
@@ -253,6 +273,23 @@ W_ReparentView(W_View *view, W_View *newParent, int x, int y)
 }
 
 
+void
+W_RaiseView(W_View *view)
+{
+    if (W_VIEW_REALIZED(view))
+	XRaiseWindow(W_VIEW_DISPLAY(view), W_VIEW_DRAWABLE(view));
+}
+
+
+
+void
+W_LowerView(W_View *view)
+{
+    if (W_VIEW_REALIZED(view))
+	XLowerWindow(W_VIEW_DISPLAY(view), W_VIEW_DRAWABLE(view));
+}
+
+
 
 void
 W_MapView(W_View *view)
@@ -311,10 +348,10 @@ W_UnmapView(W_View *view)
     view->flags.mapWhenRealized = 0;
     if (!view->flags.mapped)
 	return;
-  
+
     XUnmapWindow(view->screen->display, view->window);
     XFlush(view->screen->display);
-    
+
     view->flags.mapped = 0;
 }
 
@@ -324,7 +361,9 @@ W_TopLevelOfView(W_View *view)
 {
     W_View *toplevel;
     
-    for (toplevel=view; !toplevel->flags.topLevel; toplevel=toplevel->parent);
+    for (toplevel=view;
+	 toplevel && !toplevel->flags.topLevel; 
+	 toplevel=toplevel->parent);
     
     return toplevel;
 }
@@ -374,7 +413,7 @@ destroyView(W_View *view)
 	    }
 	}
     }
-    
+
     /* destroy children recursively */
     while (view->childrenList!=NULL) {
 	ptr = view->childrenList;
@@ -391,7 +430,8 @@ destroyView(W_View *view)
     W_CallDestroyHandlers(view);
 
     if (view->flags.realized) {
-	XDeleteContext(view->screen->display, view->window, ViewContext);
+	XDeleteContext(view->screen->display,
+		       view->window, ViewContext);
     
 	/* if parent is being destroyed, it will die naturaly */
 	if (!view->flags.parentDying || view->flags.topLevel)
@@ -401,16 +441,24 @@ destroyView(W_View *view)
     /* remove self from parent's children list */
     unparentView(view);
 
-    W_CleanUpEvents(view);
+    /* the array has a wfree() destructor that will be called automatically */
+    WMFreeArray(view->eventHandlers);
+    view->eventHandlers = NULL;
     
     WMUnregisterViewDraggedTypes(view);
     
-#if 0    
+    WMRemoveNotificationObserver(view);
+    
+#if 0
     if (view->dragSourceProcs)
 	wfree(view->dragSourceProcs);
 
     if (view->dragDestinationProcs)
 	wfree(view->dragDestinationProcs);
+    
+    if (scr->dragInfo.destView == view) {
+	scr->dragInfo.destView = NULL;
+    }
 #endif
     wfree(view);
 }
@@ -420,7 +468,11 @@ destroyView(W_View *view)
 void
 W_DestroyView(W_View *view)
 {
-    W_ReleaseView(view);
+    view->refCount--;
+
+    if (view->refCount < 1) {
+        destroyView(view);
+    }
 }
 
 
@@ -494,6 +546,8 @@ W_RedisplayView(W_View *view)
     ev.xexpose.window = view->window;
     ev.xexpose.count = 0;
 
+    ev.xexpose.serial = 0;
+
     WMHandleEvent(&ev);
 }
 
@@ -515,6 +569,18 @@ W_SetViewBackgroundColor(W_View *view, WMColor *color)
 }
 
 
+void 
+W_SetViewCursor(W_View *view, Cursor cursor)
+{
+    view->cursor = cursor;
+    if (W_VIEW_REALIZED(view)) {
+	XDefineCursor(W_VIEW_DISPLAY(view), W_VIEW_DRAWABLE(view), cursor);
+    } else {
+	view->attribFlags |= CWCursor;
+	view->attribs.cursor = cursor;
+    }
+}
+
 
 W_View*
 W_FocusedViewOfToplevel(W_View *view)
@@ -603,15 +669,16 @@ WMView*
 W_RetainView(WMView *view)
 {
     view->refCount++;
+
     return view;
 }
 
 
-
 void
 W_ReleaseView(WMView *view)
 {
     view->refCount--;
+
     if (view->refCount < 1) {
 	destroyView(view);
     }
@@ -641,9 +708,10 @@ WMGetViewPosition(WMView *view)
 void
 WMSetViewNotifySizeChanges(WMView *view, Bool flag)
 {
-    view->flags.notifySizeChanged = flag;
+    view->flags.notifySizeChanged = ((flag==0) ? 0 : 1);
 }
 
+
 Window
 WMViewXID(WMView *view)
 {
@@ -656,13 +724,56 @@ WMGetViewScreenPosition(WMView *view)
 {
     WMScreen *scr = W_VIEW_SCREEN(view);
     Window foo;
-    int x, y;
-    
+    int x, y, topX, topY, bar;
+    WMView *topView;
+
+    topView = view;
+    while (topView->parent && topView->parent!=scr->rootView)
+        topView = topView->parent;
+
+    if (!XGetGeometry(scr->display, W_VIEW_DRAWABLE(topView), &foo, &topX,
+                      &topY, &bar, &bar, &bar, &bar)) {
+        topX = topY = 0;
+    }
+
     XTranslateCoordinates(scr->display, W_VIEW_DRAWABLE(view),
 			  scr->rootWin, 0, 0, &x, &y, &foo);
+
+    return wmkpoint(x-topX, y-topY);
+}
+
+
+static void
+resizedParent(void *self, WMNotification *notif)
+{
+    WMSize size = WMGetViewSize((WMView*)WMGetNotificationObject(notif));
+    WMView *view = (WMView*)self;
+
+    W_MoveView(view, view->leftOffs, view->topOffs);
+    W_ResizeView(view, size.width - (view->leftOffs + view->rightOffs),
+		   size.height - (view->topOffs + view->bottomOffs));
+}
+
+
+void
+WMSetViewExpandsToParent(WMView *view, int leftOffs, int topOffs, 
+			 int rightOffs, int bottomOffs)
+{
+    WMSize size = view->parent->size;
+
+    view->topOffs = topOffs;
+    view->bottomOffs = bottomOffs;
+    view->leftOffs = leftOffs;
+    view->rightOffs = rightOffs;
     
-    return wmkpoint(x, y);
+    WMAddNotificationObserver(resizedParent, view,
+			      WMViewSizeDidChangeNotification, 
+			      view->parent);
+    WMSetViewNotifySizeChanges(view->parent, True);
+
+    W_MoveView(view, leftOffs, topOffs);
+    W_ResizeView(view, size.width - (leftOffs + rightOffs), 
+		 size.height - (topOffs + bottomOffs));
 }
 
-			
 
