@@ -2,7 +2,7 @@
  * 
  *  WPrefs - Window Maker Preferences Program
  * 
- *  Copyright (c) 2000 Alfredo K. Kojima
+ *  Copyright (c) 2000-2002 Alfredo K. Kojima
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -76,13 +76,15 @@ typedef struct _Panel {
 
     WMFrame *commandF;
     WMTextField *commandT;	       /* command to run */
+    WMButton *browseB;
     WMButton *xtermC;		       /* inside xterm? */
-    
+
     WMFrame *pathF;
     WMTextField *pathT;
 
     WMFrame *pipeF;
     WMTextField *pipeT;
+    WMButton *pipeCacheB;
 
     WMFrame *dpathF;
     WMTextField *dpathT;
@@ -134,6 +136,7 @@ typedef struct {
 	} external;
 	struct {
 	    char *command;
+	    unsigned cached:1;
 	} pipe;
 	struct {
 	    char *directory;
@@ -246,11 +249,41 @@ icommandLClicked(WMWidget *w, void *data)
 }
 
 
+static void
+browseForFile(WMWidget *self, void *clientData)
+{
+    _Panel *panel = (_Panel*)clientData;
+    WMFilePanel *filePanel;
+    char *text, *oldprog, *newprog;
 
+    filePanel = WMGetOpenPanel(WMWidgetScreen(self));
+    text = WMGetTextFieldText(panel->commandT);
 
+    oldprog = wtrimspace(text);
+    wfree(text);
 
+    if (oldprog[0]==0 || oldprog[0]!='/') {
+        wfree(oldprog);
+        oldprog = wstrdup("/");
+    } else {
+        char *ptr = oldprog;
+        while (*ptr && !isspace(*ptr))
+            ptr++;
+        *ptr = 0;
+    }
 
+    WMSetFilePanelCanChooseDirectories(filePanel, False);
 
+    if (WMRunModalFilePanelForDirectory(filePanel, panel->parent, oldprog,
+                                        _("Select Program"), NULL)==True) {
+        newprog = WMGetFilePanelFileName(filePanel);
+        WMSetTextFieldText(panel->commandT, newprog);
+        updateMenuItem(panel, panel->currentItem, panel->commandT);
+        wfree(newprog);
+    }
+
+    wfree(oldprog);
+}
 
 
 static char*
@@ -526,7 +559,7 @@ createPanel(_Panel *p)
 	data->param.exec.command = "eterm";
 
 	data = putNewItem(panel, pad, ExecInfo, _("Run..."));
-	data->param.exec.command = "%a(Run,Type command to run)";
+	data->param.exec.command = _("%a(Run,Type command to run)");
 
 	data = putNewItem(panel, pad, ExecInfo, _("Netscape"));
 	data->param.exec.command = "netscape";
@@ -642,11 +675,18 @@ createPanel(_Panel *p)
     WMResizeWidget(panel->commandF, width, 50);
     WMMoveWidget(panel->commandF, 10, 20);
     WMSetFrameTitle(panel->commandF, _("Program to Run"));
+    WMSetFrameTitlePosition(panel->commandF, WTPAtTop);
 
     panel->commandT = WMCreateTextField(panel->commandF);
-    WMResizeWidget(panel->commandT, width - 20, 20);
+    WMResizeWidget(panel->commandT, width - 95, 20);
     WMMoveWidget(panel->commandT, 10, 20);
 
+    panel->browseB = WMCreateCommandButton(panel->commandF);
+    WMResizeWidget(panel->browseB, 70, 24);
+    WMMoveWidget(panel->browseB, width - 80, 18);
+    WMSetButtonText(panel->browseB, _("Browse"));
+    WMSetButtonAction(panel->browseB, browseForFile, panel);
+
     WMAddNotificationObserver(dataChanged, panel,
 			      WMTextDidChangeNotification,
 			      panel->commandT);
@@ -691,8 +731,8 @@ createPanel(_Panel *p)
     /* pipe */
 
     panel->pipeF = WMCreateFrame(panel->optionsF);
-    WMResizeWidget(panel->pipeF, width, 100);
-    WMMoveWidget(panel->pipeF, 10, 50);
+    WMResizeWidget(panel->pipeF, width, 155);
+    WMMoveWidget(panel->pipeF, 10, 30);
     WMSetFrameTitle(panel->pipeF, _("Command"));
 
     panel->pipeT = WMCreateTextField(panel->pipeF);
@@ -710,6 +750,14 @@ createPanel(_Panel *p)
     WMSetLabelText(label, _("Enter a command that outputs a menu\n"
 			    "definition to stdout when invoked."));
 
+    
+    panel->pipeCacheB = WMCreateSwitchButton(panel->pipeF);
+    WMResizeWidget(panel->pipeCacheB, width - 20, 40);
+    WMMoveWidget(panel->pipeCacheB, 10, 110);
+    WMSetButtonText(panel->pipeCacheB, 
+		    _("Cache menu contents after opening for\n"
+		      "the first time"));
+
     WMMapSubwidgets(panel->pipeF);
 
 
@@ -769,7 +817,7 @@ createPanel(_Panel *p)
     WMSetFrameTitle(panel->shortF, _("Keyboard Shortcut"));
 
     panel->shortT = WMCreateTextField(panel->shortF);
-    WMResizeWidget(panel->shortT, width - 20 - 170, 20);
+    WMResizeWidget(panel->shortT, width - 20 - 150, 20);
     WMMoveWidget(panel->shortT, 10, 20);
 
     WMAddNotificationObserver(dataChanged, panel,
@@ -777,14 +825,14 @@ createPanel(_Panel *p)
 			      panel->shortT);
 
     panel->sgrabB = WMCreateCommandButton(panel->shortF);
-    WMResizeWidget(panel->sgrabB, 80, 24);
-    WMMoveWidget(panel->sgrabB, width - 90, 18);
+    WMResizeWidget(panel->sgrabB, 70, 24);
+    WMMoveWidget(panel->sgrabB, width - 80, 18);
     WMSetButtonText(panel->sgrabB, _("Capture"));
     WMSetButtonAction(panel->sgrabB, sgrabClicked, panel);
 
     panel->sclearB = WMCreateCommandButton(panel->shortF);
-    WMResizeWidget(panel->sclearB, 80, 24);
-    WMMoveWidget(panel->sclearB, width - 175, 18);
+    WMResizeWidget(panel->sclearB, 70, 24);
+    WMMoveWidget(panel->sclearB, width - 155, 18);
     WMSetButtonText(panel->sclearB, _("Clear"));
     WMSetButtonAction(panel->sclearB, sgrabClicked, panel);
 
@@ -900,7 +948,8 @@ createPanel(_Panel *p)
 	
 	pos.y = WMAX(pos.y - 100, 0);
 	
-	WEditMenuShowAt(panel->menu, pos.x, pos.y);
+        if (panel->menu)
+            WEditMenuShowAt(panel->menu, pos.x, pos.y);
     }
 }
 
@@ -982,11 +1031,17 @@ parseCommand(WMPropList *item)
 	/*
 	 * dir menu, menu file
 	 * dir WITH
-	 * |pipe (TODO: ||pipe)
+	 * |pipe
 	 */
 	p = parameter;
 	while (isspace(*p) && *p) p++;
 	if (*p == '|') {
+	    if (*(p+1) == '|') {
+		p++;
+		data->param.pipe.cached = 0;
+	    } else {
+		data->param.pipe.cached = 1;
+	    }
 	    data->type = PipeInfo;
 	    data->param.pipe.command = wtrimspace(p+1);
 	} else {
@@ -1225,6 +1280,10 @@ updateMenuItem(_Panel *panel, WEditMenuItem *item, WMWidget *changedWidget)
 	    REPLACE(data->param.pipe.command,
 		    WMGetTextFieldText(panel->pipeT));
 	}
+	if (changedWidget == panel->pipeCacheB) {
+	    data->param.pipe.cached =
+		WMGetButtonSelected(panel->pipeCacheB);
+	}
 	break;
 	
      case ExternalInfo:
@@ -1289,6 +1348,7 @@ menuItemCloned(WEditMenuDelegate *delegate, WEditMenu *menu,
 
      case PipeInfo:
 	newData->param.pipe.command = DUP(data->param.pipe.command);
+	newData->param.pipe.cached = data->param.pipe.cached;
 	break;
 
      case ExternalInfo:
@@ -1414,6 +1474,7 @@ menuItemSelected(WEditMenuDelegate *delegate, WEditMenu *menu,
 
 	 case PipeInfo:
 	    WMSetTextFieldText(panel->pipeT, data->param.pipe.command);
+	    WMSetButtonSelected(panel->pipeCacheB, data->param.pipe.cached);
 	    break;
 
 	 case ExternalInfo:
@@ -1681,11 +1742,14 @@ processData(char *title, ItemData *data)
 	if (!data->param.pipe.command)
 	    return NULL;
 	WMAddToPLArray(item, pomenu);
-	s1 = wstrconcat("| ", data->param.pipe.command);
+	if (data->param.pipe.cached)
+	    s1 = wstrconcat("| ", data->param.pipe.command);
+	else
+	    s1 = wstrconcat("|| ", data->param.pipe.command);
 	WMAddToPLArray(item, WMCreatePLString(s1));
 	wfree(s1);
 	break;
-	
+
      case ExternalInfo:
 	if (!data->param.external.path)
 	    return NULL;
@@ -1807,14 +1871,16 @@ storeData(_Panel *panel)
 static void
 showMenus(_Panel *panel)
 {
-    WEditMenuUnhide(panel->menu);
+    if (panel->menu)
+        WEditMenuUnhide(panel->menu);
 }
 
 
 static void
 hideMenus(_Panel *panel)
 {
-    WEditMenuHide(panel->menu);
+    if (panel->menu)
+        WEditMenuHide(panel->menu);
 }
 
 
