@@ -2,8 +2,8 @@
  * 
  *  Window Maker window manager
  * 
- *  Copyright (c) 1997, 1998, 1999 Alfredo K. Kojima
- *  Copyright (c) 1998       Dan Pascu
+ *  Copyright (c) 1997-2002 Alfredo K. Kojima
+ *  Copyright (c) 1998-2002 Dan Pascu
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -48,6 +48,7 @@
 #include "winspector.h"
 #include "workspace.h"
 #include "wsound.h"
+#include "xinerama.h"
 
 #ifdef GNOME_STUFF
 # include "gnome.h"
@@ -178,8 +179,10 @@ wSetFocusTo(WScreen *scr, WWindow  *wwin)
     napp = wApplicationOf(wwin->main_window);
 
     /* remember last workspace where the app has been */
-    if (napp) 
-	napp->last_workspace = wwin->screen_ptr->current_workspace;
+    if (napp)  {
+        /*napp->last_workspace = wwin->screen_ptr->current_workspace;*/
+        napp->last_workspace = wwin->frame->workspace;
+    }
 
     if (wwin->flags.mapped && !WFLAGP(wwin, no_focusable)) {
 	/* install colormap if colormap mode is lock mode */
@@ -423,13 +426,13 @@ wMaximizeWindow(WWindow *wwin, int directions)
 	&& !(directions & MAX_IGNORE_XINERAMA)) {
 	WScreen *scr = wwin->screen_ptr;
 	WMRect rect;
-	
-	rect = wGetRectForHead(scr, wGetHeadForWindow(wwin));
+
+	rect = wGetRectForHead(scr, wGetHeadForPointerLocation(scr));
 	totalArea.x1 = rect.pos.x;
 	totalArea.y1 = rect.pos.y;
 	totalArea.x2 = totalArea.x1 + rect.size.width;
 	totalArea.y2 = totalArea.y1 + rect.size.height;
-	
+
 	usableArea.x1 = WMAX(totalArea.x1, usableArea.x1);
 	usableArea.y1 = WMAX(totalArea.y1, usableArea.y1);
 	usableArea.x2 = WMIN(totalArea.x2, usableArea.x2);
@@ -488,7 +491,7 @@ wMaximizeWindow(WWindow *wwin, int directions)
     
     wWindowCropSize(wwin, usableArea.x2-usableArea.x1, 
 		    usableArea.y2-usableArea.y1,
-		    &new_width, &new_height);    
+		    &new_width, &new_height);
     
     wWindowConfigure(wwin, new_x, new_y, new_width, new_height);
 
@@ -899,8 +902,8 @@ wIconifyWindow(WWindow *wwin)
 	return;
     }
 
-    
-    if (wwin->transient_for!=None) {
+    if (wwin->transient_for!=None &&
+        wwin->transient_for!=wwin->screen_ptr->root_win) {
 	WWindow *owner = wWindowFor(wwin->transient_for);
 	
 	if (owner && owner->flags.miniaturized)
@@ -1071,7 +1074,7 @@ wDeiconifyWindow(WWindow  *wwin)
     if (!wwin->flags.shaded)
 	wwin->flags.mapped = 1;
 
-    if (!wPreferences.disable_miniwindows) {
+    if (!wPreferences.disable_miniwindows && wwin->icon != NULL) {
 	if (wwin->icon->selected)
 	    wIconSelect(wwin->icon);
 
@@ -1083,7 +1086,7 @@ wDeiconifyWindow(WWindow  *wwin)
     /* if the window is in another workspace, do it silently */
 #ifdef ANIMATIONS
     if (!wwin->screen_ptr->flags.startup && !wPreferences.no_animations
-	&& !wwin->flags.skip_next_animation) {
+	&& !wwin->flags.skip_next_animation && wwin->icon != NULL) {
 	int ix, iy, iw, ih;
 
 	if (!wPreferences.disable_miniwindows) {
@@ -1127,7 +1130,7 @@ wDeiconifyWindow(WWindow  *wwin)
     }
     mapTransientsFor(wwin);
 
-    if (!wPreferences.disable_miniwindows) {
+    if (!wPreferences.disable_miniwindows && wwin->icon != NULL) {
 	RemoveFromStackList(wwin->icon->core);
 	/*    removeIconGrabs(wwin->icon);*/
 	wIconDestroy(wwin->icon);
