@@ -2,8 +2,8 @@
  * 
  *  Window Maker window manager
  * 
- *  Copyright (c) 1997, 1998 Alfredo K. Kojima
- *  Copyright (c) 1998       Dan Pascu
+ *  Copyright (c) 1997-2002 Alfredo K. Kojima
+ *  Copyright (c) 1998-2002 Dan Pascu
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -176,11 +176,9 @@ static int setMultiByte();
 #endif
 static int updateUsableArea();
 
-#ifdef DEFINABLE_CURSOR
 extern Cursor wCursor[WCUR_LAST];
 static int getCursor();
 static int setCursor();
-#endif
 
 
 /*
@@ -683,6 +681,9 @@ WDefaultEntry optionList[] = {
     {"HideKey", "None",	       			(void*)WKBD_HIDE,
 	  NULL,				getKeybind,	setKeyGrab
     },
+    {"HideOthersKey", "None",	       		(void*)WKBD_HIDE_OTHERS,
+	  NULL,				getKeybind,	setKeyGrab
+    },
     {"MoveResizeKey", "None",	       		(void*)WKBD_MOVERESIZE,
 	  NULL,				getKeybind,	setKeyGrab
     },
@@ -793,18 +794,18 @@ WDefaultEntry optionList[] = {
     },
     {"ScreenSwitchKey",  "None",		(void*)WKBD_SWITCH_SCREEN,
 	    NULL, 			getKeybind,	setKeyGrab	
-    }
+    },
 
 #ifdef KEEP_XKB_LOCK_STATUS
-    ,{"ToggleKbdModeKey", "None",                      (void*)WKBD_TOGGLE,
+    {"ToggleKbdModeKey", "None",                      (void*)WKBD_TOGGLE,
 	    NULL,                       getKeybind,     setKeyGrab
     },
     {"KbdModeLock", "NO",  	                  NULL,
 	    &wPreferences.modelock, 	getBool,	NULL
-    }
+    },
 #endif /* KEEP_XKB_LOCK_STATUS */
-#ifdef DEFINABLE_CURSOR
-    ,{"NormalCursor", "(builtin, left_ptr)",	(void*)WCUR_ROOT,
+
+    {"NormalCursor", "(builtin, left_ptr)",	(void*)WCUR_ROOT,
           NULL,				getCursor,	setCursor
     },
     {"ArrowCursor", "(builtin, top_left_arrow)",	(void*)WCUR_ARROW,
@@ -852,7 +853,6 @@ WDefaultEntry optionList[] = {
     {"SelectCursor", "(builtin, cross)",	(void*)WCUR_SELECT,
           NULL,				getCursor,	setCursor
     }
-#endif /* DEFINABLE_CURSOR */
 };
 
 
@@ -908,6 +908,29 @@ initDefaults()
 }
 
 
+static WMPropList*
+readGlobalDomain(char *domainName, Bool requireDictionary)
+{
+    WMPropList *globalDict = NULL;
+    char path[PATH_MAX];
+    struct stat stbuf;
+
+    snprintf(path, sizeof(path), "%s/WindowMaker/%s", SYSCONFDIR, domainName);
+    if (stat(path, &stbuf)>=0) {
+        globalDict = WMReadPropListFromFile(path);
+        if (globalDict && requireDictionary && !WMIsPLDictionary(globalDict)) {
+            wwarning(_("Domain %s (%s) of global defaults database is corrupted!"),
+                     domainName, path);
+            WMReleasePropList(globalDict);
+            globalDict = NULL;
+        } else if (!globalDict) {
+            wwarning(_("could not load domain %s from global defaults database"),
+                     domainName);
+        }
+    }
+
+    return globalDict;
+}
 
 
 #if 0
@@ -992,7 +1015,7 @@ wDefaultsInitDomain(char *domain, Bool requireDictionary)
 	    } else {
 		if (db->dictionary && WMIsPLDictionary(shared_dict) &&
 		    WMIsPLDictionary(db->dictionary)) {
-		    WMMergePLDictionaries(shared_dict, db->dictionary);
+                    WMMergePLDictionaries(shared_dict, db->dictionary, True);
 		    WMReleasePropList(db->dictionary);
 		    db->dictionary = shared_dict;
 		    if (stbuf.st_mtime > db->timestamp)
@@ -1051,35 +1074,22 @@ wDefaultsCheckDomains(void *foo)
 {
     WScreen *scr;
     struct stat stbuf;
+    WMPropList *shared_dict = NULL;
     WMPropList *dict;
     int i;
-    char path[PATH_MAX];
 
 #ifdef HEARTBEAT
     puts("Checking domains...");
 #endif
     if (stat(WDWindowMaker->path, &stbuf)>=0
 	&& WDWindowMaker->timestamp < stbuf.st_mtime) {
-	WMPropList *shared_dict = NULL;
 #ifdef HEARTBEAT
 	puts("Checking WindowMaker domain");
 #endif
 	WDWindowMaker->timestamp = stbuf.st_mtime;	
 
 	/* global dictionary */
-	snprintf(path, sizeof(path), "%s/WindowMaker/WindowMaker", SYSCONFDIR);
-	if (stat(path, &stbuf)>=0) {
-	    shared_dict = WMReadPropListFromFile(path);
-	    if (shared_dict && !WMIsPLDictionary(shared_dict)) {
-		wwarning(_("Domain %s (%s) of global defaults database is corrupted!"),
-			 "WindowMaker", path);
-		WMReleasePropList(shared_dict);
-		shared_dict = NULL;
-	    } else if (!shared_dict) {
-		wwarning(_("could not load domain %s from global defaults database"),
-			 "WindowMaker");
-	    }
-	}
+        shared_dict = readGlobalDomain("WindowMaker", True);
 	/* user dictionary */
 	dict = WMReadPropListFromFile(WDWindowMaker->path);
 	if (dict) {
@@ -1090,7 +1100,7 @@ wDefaultsCheckDomains(void *foo)
 			 "WindowMaker", WDWindowMaker->path);
 	    } else {
 		if (shared_dict) {
-		    WMMergePLDictionaries(shared_dict, dict);
+                    WMMergePLDictionaries(shared_dict, dict, True);
 		    WMReleasePropList(dict);
 		    dict = shared_dict;
 		    shared_dict = NULL;
@@ -1119,6 +1129,9 @@ wDefaultsCheckDomains(void *foo)
 #ifdef HEARTBEAT
 	puts("Checking WMWindowAttributes domain");
 #endif
+	/* global dictionary */
+        shared_dict = readGlobalDomain("WMWindowAttributes", True);
+	/* user dictionary */
 	dict = WMReadPropListFromFile(WDWindowAttributes->path);
 	if (dict) {
 	    if (!WMIsPLDictionary(dict)) {
@@ -1127,8 +1140,15 @@ wDefaultsCheckDomains(void *foo)
 		wwarning(_("Domain %s (%s) of defaults database is corrupted!"),
 			 "WMWindowAttributes", WDWindowAttributes->path);
 	    } else {
-		if (WDWindowAttributes->dictionary)
-		    WMReleasePropList(WDWindowAttributes->dictionary);
+		if (shared_dict) {
+                    WMMergePLDictionaries(shared_dict, dict, True);
+		    WMReleasePropList(dict);
+		    dict = shared_dict;
+		    shared_dict = NULL;
+		}
+                if (WDWindowAttributes->dictionary) {
+                    WMReleasePropList(WDWindowAttributes->dictionary);
+                }
 		WDWindowAttributes->dictionary = dict;
 		for (i=0; i<wScreenCount; i++) {
 		    scr = wScreenWithNumber(i);
@@ -1157,6 +1177,9 @@ wDefaultsCheckDomains(void *foo)
 		     "WMWindowAttributes");
 	}
 	WDWindowAttributes->timestamp = stbuf.st_mtime;
+	if (shared_dict) {
+	    WMReleasePropList(shared_dict);
+	}
     }
 
 #ifndef LITE
@@ -2472,8 +2495,6 @@ getRImages(WScreen *scr, WDefaultEntry *entry, WMPropList *value,
 }
 #endif
 
-#ifdef DEFINABLE_CURSOR
-
 # include <X11/cursorfont.h>
 typedef struct 
 {
@@ -2736,8 +2757,6 @@ again:
 }
 #undef CURSOR_ID_NONE
 
-#endif /* DEFINABLE_CURSOR */
-
 
 /* ---------------- value setting functions --------------- */
 static int
@@ -3435,28 +3454,26 @@ setMultiByte(WScreen *scr, WDefaultEntry *entry, char *value, void *foo)
     extern _WINGsConfiguration WINGsConfiguration;
 
     WINGsConfiguration.useMultiByte = *value;
-    
+
     return 0;
 }
 #endif
 
-#ifdef DEFINABLE_CURSOR
+
 static int
 setCursor(WScreen *scr, WDefaultEntry *entry, Cursor *cursor, long index)
 {
-   if (None != wCursor[index])
-    {
+   if (wCursor[index] != None) {
        XFreeCursor(dpy, wCursor[index]);
-    }
-   
+   }
+
    wCursor[index] = *cursor;
 
-   if ((WCUR_ROOT == index) && (None != *cursor))
-    {
+   if (index==WCUR_ROOT && *cursor!=None) {
        XDefineCursor(dpy, scr->root_win, *cursor);
-    }
+   }
 
    return 0;
 }
-#endif /* DEFINABLE_CURSOR*/
+
 
