@@ -2,7 +2,7 @@
  * 
  *  Window Maker window manager
  * 
- *  Copyright (c) 1997, 1998 Alfredo K. Kojima
+ *  Copyright (c) 1997-2002 Alfredo K. Kojima
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -556,23 +556,24 @@ handleDestroyNotify(XEvent *event)
 	wUnmanageWindow(wwin, False, True);
     }
 
-    while ((index = WMFindInArray(scr->fakeGroupLeaders, matchWindow,
-                                  (void*)window)) != WANotFound) {
-        WFakeGroupLeader *fPtr;
+    if (scr != NULL) {
+        while ((index = WMFindInArray(scr->fakeGroupLeaders, matchWindow,
+                                      (void*)window)) != WANotFound) {
+            WFakeGroupLeader *fPtr;
 
-        fPtr = WMGetFromArray(scr->fakeGroupLeaders, index);
-        if (fPtr->retainCount > 0) {
-            fPtr->retainCount--;
-            if (fPtr->retainCount==0 && fPtr->leader!=None) {
-                XDestroyWindow(dpy, fPtr->leader);
-                fPtr->leader = None;
-                XFlush(dpy);
+            fPtr = WMGetFromArray(scr->fakeGroupLeaders, index);
+            if (fPtr->retainCount > 0) {
+                fPtr->retainCount--;
+                if (fPtr->retainCount==0 && fPtr->leader!=None) {
+                    XDestroyWindow(dpy, fPtr->leader);
+                    fPtr->leader = None;
+                    XFlush(dpy);
+                }
             }
+            fPtr->origLeader = None;
         }
-        fPtr->origLeader = None;
     }
 
-
     app = wApplicationOf(window);
     if (app) {
 	if (window == app->main_window) {
@@ -1364,8 +1365,8 @@ handleKeyPress(XEvent *event)
 	    break;
 	}
     }
-        
-    
+
+
     if (command < 0) {
 #ifdef LITE
 	{ 
@@ -1436,6 +1437,13 @@ handleKeyPress(XEvent *event)
 	    }
 	}
 	break;
+     case WKBD_HIDE_OTHERS:
+	if (ISMAPPED(wwin) && ISFOCUSED(wwin)) {
+	    CloseWindowMenu(scr);
+	    
+            wHideOtherApplications(wwin);
+	}
+	break;
      case WKBD_MAXIMIZE:
 	if (ISMAPPED(wwin) && ISFOCUSED(wwin) && !WFLAGP(wwin, no_resizable)) {
 	    CloseWindowMenu(scr);
