@@ -2,7 +2,7 @@
  * 
  *  Window Maker window manager
  * 
- *  Copyright (c) 1997, 1998 Alfredo K. Kojima
+ *  Copyright (c) 1997-2002 Alfredo K. Kojima
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -385,7 +385,7 @@ wIconChangeImage(WIcon *icon, RImage *new_image)
     if (icon->image)
         RReleaseImage(icon->image);
 
-    icon->image = wIconValidateIconSize(icon->core->screen_ptr, new_image);
+    icon->image = new_image;
 
     wIconUpdate(icon);
 }
@@ -445,6 +445,8 @@ wIconChangeImageFile(WIcon *icon, char *file)
     path = FindImage(wPreferences.icon_path, file);
 
     if (path && (image = RLoadImage(scr->rcontext, path, 0))) {
+	image = wIconValidateIconSize(icon->core->screen_ptr, image);
+
 	wIconChangeImage(icon, image);
     } else {
 	error = 1;
