@@ -2,8 +2,8 @@
  *
  *  Window Maker window manager
  * 
- *  Copyright (c) 1997, 1998 Alfredo K. Kojima
- *  Copyright (c) 1997, 1998 Dan Pascu
+ *  Copyright (c) 1997-2002 Alfredo K. Kojima
+ *  Copyright (c) 1998-2002 Dan Pascu
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -820,10 +820,19 @@ paintEntry(WMenu *menu, int index, int selected)
 	ih = indicator->height;
 	XSetClipMask(dpy, scr->copy_gc, indicator->mask);
 	XSetClipOrigin(dpy, scr->copy_gc, 5, y+(h-ih)/2);
-	if (selected)
-	    XSetForeground(dpy, scr->copy_gc, scr->black_pixel);
-	else
-	    XSetForeground(dpy, scr->copy_gc, scr->mtext_pixel);
+        if (selected) {
+            if (entry->flags.enabled) {
+                XSetForeground(dpy, scr->copy_gc, scr->black_pixel);
+            } else {
+                XSetForeground(dpy, scr->copy_gc, scr->dtext_pixel);
+            }
+        } else {
+            if (entry->flags.enabled) {
+                XSetForeground(dpy, scr->copy_gc, scr->mtext_pixel);
+            } else {
+                XSetForeground(dpy, scr->copy_gc, scr->dtext_pixel);
+            }
+        }
 	XFillRectangle(dpy, win, scr->copy_gc, 5, y+(h-ih)/2, iw, ih);
 	/*
 	XCopyArea(dpy, indicator->image, win, scr->copy_gc, 0, 0,
