@@ -1,7 +1,7 @@
 /*
  *  Window Maker window manager
  * 
- *  Copyright (c) 1997, 1998 Alfredo K. Kojima
+ *  Copyright (c) 1997-2002 Alfredo K. Kojima
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -39,7 +39,7 @@ typedef struct WMenuEntry {
     void *clientdata;		       /* data to pass to callback */
     int cascade;		       /* cascade menu index */
 #ifdef USER_MENU
-	proplist_t instances;	/* allowed instances */
+    WMPropList *instances;	       /* allowed instances */
 #endif /* USER_MENU */
     struct {
 	unsigned int enabled:1;	       /* entry is selectable */
