@@ -2,7 +2,7 @@
  * 
  *  Window Maker window manager
  * 
- *  Copyright (c) 1997, 1998 Alfredo K. Kojima
+ *  Copyright (c) 1997-2002 Alfredo K. Kojima
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -247,7 +247,7 @@ PlaceIcon(WScreen *scr, int *x_ret, int *y_ret)
 	    break;
     }
 
-    free(map);
+    wfree(map);
 }
 
 
@@ -467,7 +467,8 @@ autoPlaceWindow(WWindow *wwin, int *x_ret, int *y_ret,
 		if ((tx < (test_x + width)) && ((tx + tw) > test_x) &&
 		    (ty < (test_y + height)) && ((ty + th) > test_y) &&
 		    (test_window->flags.mapped ||
-		     (test_window->flags.shaded &&
+                     (test_window->flags.shaded &&
+                      test_window->frame->workspace==scr->current_workspace &&
 		      !(test_window->flags.miniaturized ||
 			test_window->flags.hidden)))) {
 
@@ -502,6 +503,7 @@ autoPlaceWindow(WWindow *wwin, int *x_ret, int *y_ret,
 		    (ty < (test_y + height)) && ((ty + th) > test_y) &&
 		    (test_window->flags.mapped ||
 		     (test_window->flags.shaded &&
+                      test_window->frame->workspace==scr->current_workspace &&
 		      !(test_window->flags.miniaturized ||
 			test_window->flags.hidden)))) {
 
