@@ -1,7 +1,7 @@
 /*
  *  Window Maker window manager
  * 
- *  Copyright (c) 1997, 1998 Alfredo K. Kojima
+ *  Copyright (c) 1997-2002 Alfredo K. Kojima
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -250,7 +250,7 @@ PropSetIconTileHint(WScreen *scr, RImage *image)
     XChangeProperty(dpy, scr->info_window, _XA_WINDOWMAKER_ICON_TILE,
 		    imageAtom, 8, PropModeReplace, tmp, 
 		    image->width * image->height * 4 + 4);
-    free(tmp);
+    wfree(tmp);
     
 }
 
