@@ -2,7 +2,7 @@
  * 
  *  Window Maker window manager
  * 
- *  Copyright (c) 1997, 1998 Alfredo K. Kojima
+ *  Copyright (c) 1997-2002 Alfredo K. Kojima
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -63,6 +63,8 @@
 # include "openlook.h"
 #endif
 
+#include "xinerama.h"
+
 #include <WINGs/WUtil.h>
 
 #include "defaults.h"
@@ -698,6 +700,11 @@ wScreenInit(int screen_number)
 	wfree(scr);
 	return NULL;
     }
+    
+#ifdef XINERAMA
+    wInitXinerama(scr);
+#endif
+    
 
     XDefineCursor(dpy, scr->root_win, wCursor[WCUR_ROOT]);
 
@@ -1018,9 +1025,9 @@ wScreenRestoreState(WScreen *scr)
 
     make_keys();
 
-    if (wScreenCount == 1)
+    if (wScreenCount == 1) {
 	path = wdefaultspathfordomain("WMState");
-    else {
+    } else {
 	char buf[16];
 	snprintf(buf, sizeof(buf), "WMState.%i", scr->screen);
 	path = wdefaultspathfordomain(buf);
@@ -1028,14 +1035,12 @@ wScreenRestoreState(WScreen *scr)
     scr->session_state = WMReadPropListFromFile(path);
     wfree(path);
     if (!scr->session_state && wScreenCount>1) {
-	char buf[16];
-	snprintf(buf, sizeof(buf), "WMState.%i", scr->screen);
-	path = wdefaultspathfordomain(buf);
+	path = wdefaultspathfordomain("WMState");
 	scr->session_state = WMReadPropListFromFile(path);
 	wfree(path);
     }
 
-    if (!wPreferences.flags.noclip) {
+    if (!wPreferences.flags.noclip && scr->session_state) {
         state = WMGetFromPLDictionary(scr->session_state, dClip);
         scr->clip_icon = wClipRestoreState(scr, state);
     }
@@ -1049,7 +1054,7 @@ wScreenRestoreState(WScreen *scr)
         wWorkspaceManageEdge(scr);
 #endif
 
-    if (!wPreferences.flags.nodock) {
+    if (!wPreferences.flags.nodock && scr->session_state) {
         state = WMGetFromPLDictionary(scr->session_state, dDock);
         scr->dock = wDockRestoreState(scr, state, WM_DOCK);
     }
