@@ -1,7 +1,7 @@
 /* session.c - session state handling and R6 style session management
  *
- *  Copyright (c) 1998 Dan Pascu
- *  Copyright (c) 1998, 1999 Alfredo Kojima
+ *  Copyright (c) 1998-2002 Dan Pascu
+ *  Copyright (c) 1998-2002 Alfredo Kojima
  *
  *  Window Maker window manager
  *
@@ -220,8 +220,8 @@ makeWindowState(WWindow *wwin, WApplication *wapp)
     WMPropList *shaded, *miniaturized, *hidden, *geometry;
     WMPropList *dock, *shortcut;
 
-    if (wwin->main_window!=None && wwin->main_window!=wwin->client_win)
-        win = wwin->main_window;
+    if (wwin->orig_main_window!=None && wwin->orig_main_window!=wwin->client_win)
+        win = wwin->orig_main_window;
     else
         win = wwin->client_win;
 
@@ -332,9 +332,11 @@ wSessionSaveState(WScreen *scr)
 
     while (wwin) {
         WApplication *wapp=wApplicationOf(wwin->main_window);
+        Window appId = wwin->orig_main_window;
 
-        if (wwin->transient_for==None 
-	    && WMGetFirstInArray(wapp_list, wapp)==WANotFound
+        if ((wwin->transient_for==None
+             || wwin->transient_for==wwin->screen_ptr->root_win)
+            && WMGetFirstInArray(wapp_list, (void*)appId)==WANotFound
 	    && !WFLAGP(wwin, dont_save_session)) {
             /* A entry for this application was not yet saved. Save one. */
             if ((win_info = makeWindowState(wwin, wapp))!=NULL) {
@@ -345,7 +347,7 @@ wSessionSaveState(WScreen *scr)
                  * application list, so no multiple entries for the same
                  * application are saved.
                  */
-		WMAddToArray(wapp_list, wapp);
+		WMAddToArray(wapp_list, (void*)appId);
             }
         }
         wwin = wwin->prev;
