@@ -1,6 +1,6 @@
 /*  session.h
  *
- *  Copyright (c) 1998 Dan Pascu
+ *  Copyright (c) 1999-2002 Alfredo K. Kojima
  *
  *  Window Maker window manager
  *
@@ -23,6 +23,28 @@
 #ifndef WMSESSION_H_
 #define WMSESSION_H_
 
+
+typedef struct {
+    int x;
+    int y;
+    unsigned int w;		       /* client size */
+    unsigned int h;
+
+    int workspace;
+    unsigned shortcuts; /* mask like 1<<shortcut_number */
+
+    WWindowAttributes mflags;
+    WWindowAttributes flags;
+
+    char miniaturized;
+    char shaded;
+    char maximized;
+
+    char user_changed_width;
+    char user_changed_height;
+} WSessionData;
+
+
 void wSessionSaveState(WScreen *scr);
 
 void wSessionSaveClients(WScreen *scr);
@@ -46,4 +68,6 @@ Bool  wSessionIsManaged(void);
 
 #endif
 
+Bool wSessionGetStateFor(WWindow *wwin, WSessionData *state);
+
 #endif
