@@ -1,8 +1,8 @@
 /*
  *  Window Maker window manager
  *
- *  Copyright (c) 1997, 1998 Alfredo K. Kojima
- *  Copyright (c) 1999       Dan Pascu
+ *  Copyright (c) 1997-2002 Alfredo K. Kojima
+ *  Copyright (c) 1998-2002 Dan Pascu
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -279,7 +279,7 @@ handleExitSig(int sig)
          * the select() is returned becaused of the signal, even if
          * there are no X events in the queue */
         WDelayedActionSet++;
-    } else if (sig == SIGINT || sig == SIGHUP) {
+    } else if (sig==SIGTERM || sig==SIGINT || sig==SIGHUP) {
 #ifdef SYS_SIGLIST_DECLARED
         wwarning(_("got signal %i (%s) - exiting...\n"), sig, sys_siglist[sig]);
 #else
@@ -313,7 +313,7 @@ handleSig(int sig)
     static int already_crashed = 0;
     int dumpcore = 0;
 #ifndef NO_EMERGENCY_AUTORESTART
-    int crashAction;
+    int crashAction, i;
     char *argv[2];
     
     argv[1] = NULL;
@@ -362,7 +362,6 @@ handleSig(int sig)
         if (dpy) {
             CARD32 data[2];
             WWindow *wwin;
-            int i;
 
 	    XGrabServer(dpy);
             crashAction = wShowCrashingDialogPanel(sig);
@@ -411,9 +410,10 @@ handleSig(int sig)
 
     	wmessage(_("trying to start alternate window manager..."));
 
-        Restart(FALLBACK_WINDOWMANAGER, False);
-        Restart("fvwm", False);
-        Restart("twm", False);
+        for (i=0; i<WMGetArrayItemCount(wPreferences.fallbackWMs); i++) {
+            Restart(WMGetFromArray(wPreferences.fallbackWMs, i), False);
+        }
+
         wfatal(_("failed to start alternate window manager. Aborting."));
 #else
         wfatal(_("a fatal error has occured, probably due to a bug. "
@@ -805,11 +805,7 @@ StartUp(Bool defaultScreenOnly)
 
 
     /* cursors */
-#ifdef DEFINABLE_CURSOR
-    wCursor[WCUR_NORMAL] = None;
-#else
-    wCursor[WCUR_NORMAL] = XCreateFontCursor(dpy, XC_left_ptr);    
-#endif
+    wCursor[WCUR_NORMAL] = None; /* inherit from root */
     wCursor[WCUR_ROOT] = XCreateFontCursor(dpy, XC_left_ptr);
     wCursor[WCUR_ARROW] = XCreateFontCursor(dpy, XC_top_left_arrow);
     wCursor[WCUR_MOVE] = XCreateFontCursor(dpy, XC_fleur);
@@ -842,12 +838,11 @@ StartUp(Bool defaultScreenOnly)
     sig_action.sa_handler = handleExitSig;
     
     /* Here we set SA_RESTART for safety, because SIGUSR1 may not be handled
-     * immediately.
-     * -Dan */
+     * immediately. -Dan */
     sig_action.sa_flags = SA_RESTART;
-/*    sigaction(SIGTERM, &sig_action, NULL);*/
-    sigaction(SIGINT, &sig_action, NULL);    
-    sigaction(SIGHUP, &sig_action, NULL);    
+    sigaction(SIGTERM, &sig_action, NULL);
+    sigaction(SIGINT, &sig_action, NULL);
+    sigaction(SIGHUP, &sig_action, NULL);
     sigaction(SIGUSR1, &sig_action, NULL);
     sigaction(SIGUSR2, &sig_action, NULL);
 
