@@ -2,7 +2,7 @@
  * 
  *  Window Maker window manager
  * 
- *  Copyright (c) 1997, 1998 Alfredo K. Kojima
+ *  Copyright (c) 1997-2002 Alfredo K. Kojima
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -34,6 +34,8 @@
 #include <stdio.h>
 #include <string.h>
 
+/* For getting mouse wheel mappings from WINGs */
+#include <WINGs/WINGsP.h>
 
 #include "WindowMaker.h"
 #include "GNUstep.h"
@@ -236,23 +238,33 @@ wWindowDestroy(WWindow *wwin)
 	}
     }
 
+    if (wwin->fake_group && wwin->fake_group->retainCount>0) {
+        wwin->fake_group->retainCount--;
+        if (wwin->fake_group->retainCount==0 && wwin->fake_group->leader!=None) {
+            XDestroyWindow(dpy, wwin->fake_group->leader);
+            wwin->fake_group->leader = None;
+            wwin->fake_group->origLeader = None;
+            XFlush(dpy);
+        }
+    }
+
     if (wwin->normal_hints)
-      XFree(wwin->normal_hints);
-    
+        XFree(wwin->normal_hints);
+
     if (wwin->wm_hints)
-      XFree(wwin->wm_hints);
-    
+        XFree(wwin->wm_hints);
+
     if (wwin->wm_instance)
-      XFree(wwin->wm_instance);
+        XFree(wwin->wm_instance);
 
     if (wwin->wm_class)
-      XFree(wwin->wm_class);
+        XFree(wwin->wm_class);
 
     if (wwin->wm_gnustep_attr)
-      wfree(wwin->wm_gnustep_attr);
+        wfree(wwin->wm_gnustep_attr);
 
     if (wwin->cmap_windows)
-      XFree(wwin->cmap_windows);
+        XFree(wwin->cmap_windows);
 
     XDeleteContext(dpy, wwin->client_win, wWinContext);
 
@@ -378,17 +390,17 @@ wWindowSetupInitialAttributes(WWindow *wwin, int *level, int *workspace)
 	WSETUFLAG(wwin, kill_close, 1);
 
     /* transients can't be iconified or maximized */
-    if (wwin->transient_for) {
-	WSETUFLAG(wwin, no_miniaturizable, 1);
+    if (wwin->transient_for!=None && wwin->transient_for!=scr->root_win) {
+        WSETUFLAG(wwin, no_miniaturizable, 1);
 	WSETUFLAG(wwin, no_miniaturize_button, 1);
     }
 
     /* if the window can't be resized, remove the resizebar */
     if (wwin->normal_hints->flags & (PMinSize|PMaxSize)
-	&& (wwin->normal_hints->min_width==wwin->normal_hints->max_width)
-	&& (wwin->normal_hints->min_height==wwin->normal_hints->max_height)) {
-	WSETUFLAG(wwin, no_resizable, 1);
-	WSETUFLAG(wwin, no_resizebar, 1);
+        && (wwin->normal_hints->min_width==wwin->normal_hints->max_width)
+        && (wwin->normal_hints->min_height==wwin->normal_hints->max_height)) {
+        WSETUFLAG(wwin, no_resizable, 1);
+        WSETUFLAG(wwin, no_resizebar, 1);
     }
     
     /* set GNUstep window attributes */
@@ -462,22 +474,34 @@ wWindowSetupInitialAttributes(WWindow *wwin, int *level, int *workspace)
      */
     if (wwin->user_flags.no_appicon && wwin->defined_user_flags.no_appicon)
 	wwin->user_flags.emulate_appicon = 0;
+        /*//WSETUFLAG(wwin, emulate_appicon, 0);*/
 
     if (wwin->main_window!=None) {
 	WApplication *wapp = wApplicationOf(wwin->main_window);
 	if (wapp && !wapp->flags.emulated)
 	    wwin->user_flags.emulate_appicon = 0;
+            /*//WSETUFLAG(wwin, emulate_appicon, 0);*/
     }
-    
-    if (wwin->transient_for!=None 
+
+    if (wwin->transient_for!=None
 	&& wwin->transient_for!=wwin->screen_ptr->root_win)
 	wwin->user_flags.emulate_appicon = 0;
-    
-    if (wwin->user_flags.sunken && wwin->defined_user_flags.sunken 
+        /*//WSETUFLAG(wwin, emulate_appicon, 0);*/
+
+    if (wwin->user_flags.sunken && wwin->defined_user_flags.sunken
 	&& wwin->user_flags.floating && wwin->defined_user_flags.floating)
 	wwin->user_flags.sunken = 0;
+        /*//WSETUFLAG(wwin, sunken, 0);*/
 
     WSETUFLAG(wwin, no_shadeable, WFLAGP(wwin, no_titlebar));
+    
+    
+    /* windows that have takefocus=False shouldn't take focus at all */
+    if (wwin->focus_mode == WFM_NO_INPUT) {
+	/* dont use WSETUFLAG, since this was not an attribute change 
+	 * made by the user */
+	wwin->user_flags.no_focusable = 1;
+    }
 }
 
 
@@ -522,6 +546,50 @@ wWindowObscuresWindow(WWindow *wwin, WWindow *obscured)
 }
 
 
+static Window
+createFakeWindowGroupLeader(WScreen *scr, Window win, char *instance, char *class)
+{
+    XClassHint *classHint;
+    XWMHints *hints;
+    Window leader;
+    int argc;
+    char **argv;
+
+    leader = XCreateSimpleWindow(dpy, scr->root_win, 10, 10, 10, 10, 0, 0, 0);
+    /* set class hint */
+    classHint = XAllocClassHint();
+    classHint->res_name = instance;
+    classHint->res_class = class;
+    XSetClassHint(dpy, leader, classHint);
+    XFree(classHint);
+
+    /* inherit these from the original leader if available */
+    hints = XGetWMHints(dpy, win);
+    if (!hints) {
+        hints = XAllocWMHints();
+        hints->flags = 0;
+    }
+    /* set window group leader to self */
+    hints->window_group = leader;
+    hints->flags |= WindowGroupHint;
+    XSetWMHints(dpy, leader, hints);
+    XFree(hints);
+
+    if (XGetCommand(dpy, win, &argv, &argc)!=0 && argc > 0) {
+        XSetCommand(dpy, leader, argv, argc);
+        XFreeStringList(argv);
+    }
+
+    return leader;
+}
+
+
+static int
+matchIdentifier(void *item, void *cdata)
+{
+    return (strcmp(((WFakeGroupLeader*)item)->identifier, (char*)cdata)==0);
+}
+
 
 /*
  *----------------------------------------------------------------
@@ -564,6 +632,7 @@ wManageWindow(WScreen *scr, Window window)
     int workspace = -1;
     char *title;
     Bool withdraw = False;
+    Bool raise = False;
 
     /* mutex. */
     /* XGrabServer(dpy); */
@@ -687,7 +756,7 @@ wManageWindow(WScreen *scr, Window window)
     }
 
     PropGetProtocols(window, &wwin->protocols);
-    
+
     if (!XGetTransientForHint(dpy, window, &wwin->transient_for)) {
 	wwin->transient_for = None;
     } else {
@@ -741,13 +810,111 @@ wManageWindow(WScreen *scr, Window window)
     }
 #endif /* OLWM_HINTS */
 
-    /*
-     * Make broken apps behave as a nice app.
-     */
+    /* Make broken apps behave as a nice app. */
     if (WFLAGP(wwin, emulate_appicon)) {
 	wwin->main_window = wwin->client_win;
     }
 
+    wwin->orig_main_window = wwin->main_window;
+
+    if (wwin->flags.is_gnustep) {
+        WSETUFLAG(wwin, shared_appicon, 0);
+    }
+
+    if (wwin->main_window) {
+        extern Atom _XA_WINDOWMAKER_MENU;
+        XTextProperty text_prop;
+
+        if (XGetTextProperty(dpy, wwin->main_window, &text_prop,
+                             _XA_WINDOWMAKER_MENU)) {
+            WSETUFLAG(wwin, shared_appicon, 0);
+        }
+    }
+
+    if (!withdraw && wwin->main_window && WFLAGP(wwin, shared_appicon)) {
+        char *buffer, *instance, *class;
+        WFakeGroupLeader *fPtr;
+        int index;
+
+#define ADEQUATE(x) ((x)!=None && (x)!=wwin->client_win && (x)!=fPtr->leader)
+
+	
+        PropGetWMClass(wwin->main_window, &class, &instance);
+	buffer = StrConcatDot(instance, class);
+
+        index = WMFindInArray(scr->fakeGroupLeaders, matchIdentifier, (void*)buffer);
+        if (index != WANotFound) {
+            fPtr = WMGetFromArray(scr->fakeGroupLeaders, index);
+            if (fPtr->retainCount == 0) {
+                fPtr->leader = createFakeWindowGroupLeader(scr, wwin->main_window,
+                                                           instance, class);
+            }
+            fPtr->retainCount++;
+#undef method2
+            if (fPtr->origLeader==None) {
+#ifdef method2
+                if (ADEQUATE(wwin->group_id)) {
+                    fPtr->retainCount++;
+                    fPtr->origLeader = wwin->group_id;
+                } else if (ADEQUATE(wwin->client_leader)) {
+                    fPtr->retainCount++;
+                    fPtr->origLeader = wwin->client_leader;
+                } else if (ADEQUATE(wwin->main_window)) {
+                    fPtr->retainCount++;
+                    fPtr->origLeader = wwin->main_window;
+                }
+#else
+                if (ADEQUATE(wwin->main_window)) {
+                    fPtr->retainCount++;
+                    fPtr->origLeader = wwin->main_window;
+                }
+#endif
+            }
+            wwin->fake_group = fPtr;
+            /*wwin->group_id = fPtr->leader;*/
+            wwin->main_window = fPtr->leader;
+            wfree(buffer);
+        } else {
+            fPtr = (WFakeGroupLeader*)wmalloc(sizeof(WFakeGroupLeader));
+
+            fPtr->identifier = buffer;
+            fPtr->leader = createFakeWindowGroupLeader(scr, wwin->main_window,
+                                                       instance, class);
+            fPtr->origLeader = None;
+            fPtr->retainCount = 1;
+
+            WMAddToArray(scr->fakeGroupLeaders, fPtr);
+
+#ifdef method2
+            if (ADEQUATE(wwin->group_id)) {
+                fPtr->retainCount++;
+                fPtr->origLeader = wwin->group_id;
+            } else if (ADEQUATE(wwin->client_leader)) {
+                fPtr->retainCount++;
+                fPtr->origLeader = wwin->client_leader;
+            } else if (ADEQUATE(wwin->main_window)) {
+                fPtr->retainCount++;
+                fPtr->origLeader = wwin->main_window;
+            }
+#else
+            if (ADEQUATE(wwin->main_window)) {
+                fPtr->retainCount++;
+                fPtr->origLeader = wwin->main_window;
+            }
+#endif
+            wwin->fake_group = fPtr;
+            /*wwin->group_id = fPtr->leader;*/
+            wwin->main_window = fPtr->leader;
+        }
+        if (instance)
+            XFree(instance);
+        if (class)
+            XFree(class);
+
+#undef method2
+#undef ADEQUATE
+    }
+
     /*
      *------------------------------------------------------------
      *
@@ -1009,7 +1176,7 @@ wManageWindow(WScreen *scr, Window window)
 	foo |= WFF_LEFT_BUTTON;
 #ifdef XKB_BUTTON_HINT
     if (WFLAGP(wwin, no_language_button) || WFLAGP(wwin, no_focusable))
-    foo |= WFF_LANGUAGE_BUTTON;
+        foo |= WFF_LANGUAGE_BUTTON;
 #endif
     if (foo!=0)
 	wFrameWindowHideButton(wwin->frame, foo);
@@ -1096,6 +1263,8 @@ wManageWindow(WScreen *scr, Window window)
         if (app) {
             app->last_workspace = workspace;
 
+            app->main_window_desc->fake_group = wwin->fake_group;
+
             /*
              * Do application specific stuff, like setting application
              * wide attributes.
@@ -1105,10 +1274,13 @@ wManageWindow(WScreen *scr, Window window)
                 /* if the window was set to hidden because it was hidden
                  * in a previous incarnation and that state was restored */
                 app->flags.hidden = 1;
-            }
-
-            if (app->flags.hidden) {
-                wwin->flags.hidden = 1;
+            } else if (app->flags.hidden) {
+                if (WFLAGP(app->main_window_desc, start_hidden)) {
+                    wwin->flags.hidden = 1;
+                } else {
+                    wUnhideApplication(app, False, False);
+                    raise = True;
+                }
             }
         }
     }
@@ -1159,7 +1331,8 @@ wManageWindow(WScreen *scr, Window window)
 #endif
 
 	if (wPreferences.superfluous && !wPreferences.no_animations
-	    && !scr->flags.startup && wwin->transient_for==None
+            && !scr->flags.startup &&
+            (wwin->transient_for==None || wwin->transient_for==scr->root_win)
 	    /* 
 	     * The brain damaged idiotic non-click to focus modes will
 	     * have trouble with this because:
@@ -1181,16 +1354,12 @@ wManageWindow(WScreen *scr, Window window)
 	    DoWindowBirth(wwin);
 	}
 
-	wWindowMap(wwin);
+        wWindowMap(wwin);
     }
 
     /* setup stacking descriptor */
     if (transientOwner) {
-	/* && wPreferences.on_top_transients */
-	if (transientOwner) {
-	    wwin->frame->core->stacking->child_of =
-		transientOwner->frame->core;
-	}
+        wwin->frame->core->stacking->child_of = transientOwner->frame->core;
     } else {
 	wwin->frame->core->stacking->child_of = NULL;
     }
@@ -1213,6 +1382,12 @@ wManageWindow(WScreen *scr, Window window)
 	wwin->prev = NULL;
     }
 
+    /* raise is set to true if we un-hid the app when this window was born.
+     * we raise, else old windows of this app will be above this new one. */
+    if (raise) {
+        wRaiseFrame(wwin->frame->core);
+    }
+
     /* Update name must come after WApplication stuff is done */
     wWindowUpdateName(wwin, title);
     if (title)
@@ -1393,12 +1568,7 @@ wManageInternalWindow(WScreen *scr, Window window, Window owner,
     XMapSubwindows(dpy, wwin->frame->core->window);
     
     /* setup stacking descriptor */
-    if (
-#ifdef removed
-	wPreferences.on_top_transients && 
-#endif
-	wwin->transient_for!=None
-	&& wwin->transient_for!=scr->root_win) {
+    if (wwin->transient_for!=None && wwin->transient_for!=scr->root_win) {
 	WWindow *tmp;
 	tmp = wWindowFor(wwin->transient_for);
 	if (tmp)
@@ -1735,47 +1905,23 @@ wWindowUnfocus(WWindow *wwin)
 void
 wWindowUpdateName(WWindow *wwin, char *newTitle)
 {
-    WApplication *app = wApplicationOf(wwin->main_window);
-    int instIndex = 0;
-    char prefix[32] = "";
     char *title;
     
     if (!wwin->frame)
 	return;
     
-    if (app) 
-	instIndex = wApplicationIndexOfGroup(app);
-    
-    
     wwin->flags.wm_name_changed = 1;
     
     if (!newTitle) {
 	/* the hint was removed */
 	title = DEF_WINDOW_TITLE;
-
-	WMPostNotificationName(WMNChangedName, wwin, NULL);
     } else {
 	title = newTitle;
     }
-    
-#ifndef NO_WINDOW_ENUMERATOR
-    if (instIndex > 0) {
-	snprintf(prefix, sizeof(prefix), "  [%i]", instIndex);
-	
-	title = wstrconcat(title, prefix);
-    }
-#endif
-    
-    if (wFrameWindowChangeTitle(wwin->frame, title)) {
 
+    if (wFrameWindowChangeTitle(wwin->frame, title)) {
 	WMPostNotificationName(WMNChangedName, wwin, NULL);
     }
-
-#ifndef NO_WINDOW_ENUMERATOR
-    if (instIndex > 0)
-	wfree(title);
-#endif
-
 }
 
 
@@ -2864,7 +3010,7 @@ titlebarDblClick(WCoreWindow *sender, void *data, XEvent *event)
             }
 
 	    /* maximize window */
-	    if (dir !=0 && !WFLAGP(wwin, no_resizable)) {
+	    if (dir!=0 && !WFLAGP(wwin, no_resizable)) {
 		int ndir = dir ^ wwin->flags.maximized;
 		if (wwin->flags.maximized != 0)
 		    wUnmaximizeWindow(wwin);
@@ -2878,6 +3024,10 @@ titlebarDblClick(WCoreWindow *sender, void *data, XEvent *event)
 	}
     } else if (event->xbutton.button==Button2) {
 	wSelectWindow(wwin, !wwin->flags.selected);
+    } else if (event->xbutton.button == WINGsConfiguration.mouseWheelUp) {
+        wShadeWindow(wwin);
+    } else if (event->xbutton.button == WINGsConfiguration.mouseWheelDown) {
+        wUnshadeWindow(wwin);
     }
 }
 
