@@ -2,8 +2,8 @@
  * 
  *  Window Maker window manager
  * 
- *  Copyright (c) 1997, 1998 Alfredo K. Kojima
- *  Copyright (c) 1998       Dan Pascu
+ *  Copyright (c) 1997-2002 Alfredo K. Kojima
+ *  Copyright (c) 1998-2002 Dan Pascu
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -154,7 +154,7 @@ static WMPropList *AStartMaximized;
 static WMPropList *ADontSaveSession;
 static WMPropList *AEmulateAppIcon;
 static WMPropList *AFullMaximize;
-static WMPropList *ACollapseAppIcons;
+static WMPropList *ASharedAppIcon;
 #ifdef XKB_BUTTON_HINT
 static WMPropList *ANoLanguageButton;
 #endif
@@ -222,7 +222,7 @@ make_keys()
     ADontSaveSession = WMCreatePLString("DontSaveSession");
     AEmulateAppIcon = WMCreatePLString("EmulateAppIcon");
     AFullMaximize = WMCreatePLString("FullMaximize");
-    ACollapseAppIcons = WMCreatePLString("CollapseAppIcons");    
+    ASharedAppIcon = WMCreatePLString("SharedAppIcon");
 #ifdef XKB_BUTTON_HINT
     ANoLanguageButton = WMCreatePLString("NoLanguageButton");
 #endif
@@ -512,23 +512,22 @@ saveSettings(WMButton *button, InspectorPanel *panel)
     WWindow *wwin = panel->inspected;
     WDDomain *db = WDWindowAttributes;
     WMPropList *dict = db->dictionary;
-    WMPropList *winDic, *value, *key;
+    WMPropList *winDic, *appDic, *value, *key, *key2;
     char *icon_file;
     int flags = 0;
-    int different = 0;
+    int different = 0, different2 = 0;
 
     /* Save will apply the changes and save them */
     applySettings(panel->applyBtn, panel);
-	
+
     if (WMGetButtonSelected(panel->instRb) != 0)
         key = WMCreatePLString(wwin->wm_instance);
     else if (WMGetButtonSelected(panel->clsRb) != 0)
         key = WMCreatePLString(wwin->wm_class);
     else if (WMGetButtonSelected(panel->bothRb) != 0) {
-		char *buffer;
+        char *buffer;
 
-		buffer = wmalloc(strlen(wwin->wm_instance)+strlen(wwin->wm_class)+4);
-        strcat(strcat(strcpy(buffer, wwin->wm_instance), "."), wwin->wm_class);
+	buffer = StrConcatDot(wwin->wm_instance, wwin->wm_class);
         key = WMCreatePLString(buffer);
         wfree(buffer);
     } else if (WMGetButtonSelected(panel->defaultRb) != 0) {
@@ -557,6 +556,7 @@ saveSettings(WMButton *button, InspectorPanel *panel)
     WMPLSetCaseSensitive(True);
 
     winDic = WMCreatePLDictionary(NULL, NULL, NULL);
+    appDic = WMCreatePLDictionary(NULL, NULL, NULL);
 
     /* Update icon for window */
     icon_file = WMGetTextFieldText(panel->fileText);
@@ -564,6 +564,7 @@ saveSettings(WMButton *button, InspectorPanel *panel)
         if (icon_file[0] != 0) {
             value = WMCreatePLString(icon_file);
             different |= insertAttribute(dict, winDic, AIcon, value, flags);
+            different2 |= insertAttribute(dict, appDic, AIcon, value, flags);
             WMReleasePropList(value);
         }
         wfree(icon_file);
@@ -650,19 +651,59 @@ saveSettings(WMButton *button, InspectorPanel *panel)
     different |= insertAttribute(dict, winDic, ANoLanguageButton, value, flags);
 #endif
 
-    /* application wide settings for when */
-    /* the window is the leader, save the attribute with the others */
-    if (panel->inspected->main_window == panel->inspected->client_win) {
-	
-	value = (WMGetButtonSelected(panel->appChk[0])!=0) ? Yes : No;
-	different |= insertAttribute(dict, winDic, AStartHidden, value, flags);
+    if (wwin->main_window!=None && wApplicationOf(wwin->main_window)!=NULL) {
+        value = (WMGetButtonSelected(panel->appChk[0])!=0) ? Yes : No;
+        different2 |= insertAttribute(dict, appDic, AStartHidden, value, flags);
 
-	value = (WMGetButtonSelected(panel->appChk[1])!=0) ? Yes : No;
-	different |= insertAttribute(dict, winDic, ANoAppIcon, value, flags);
-	
-	value = (WMGetButtonSelected(panel->appChk[2])!=0) ? Yes : No;
-	different |= insertAttribute(dict, winDic, ACollapseAppIcons, value, flags);
-    } 
+        value = (WMGetButtonSelected(panel->appChk[1])!=0) ? Yes : No;
+        different2 |= insertAttribute(dict, appDic, ANoAppIcon, value, flags);
+
+        value = (WMGetButtonSelected(panel->appChk[2])!=0) ? Yes : No;
+        different2 |= insertAttribute(dict, appDic, ASharedAppIcon, value, flags);
+    }
+
+    if (wwin->fake_group) {
+        key2 = WMCreatePLString(wwin->fake_group->identifier);
+        if (WMIsPropListEqualTo(key, key2)) {
+            WMMergePLDictionaries(winDic, appDic, True);
+            different |= different2;
+        } else {
+            WMRemoveFromPLDictionary(dict, key2);
+            if (different2) {
+                WMPutInPLDictionary(dict, key2, appDic);
+            }
+        }
+        WMReleasePropList(key2);
+        WMReleasePropList(appDic);
+    } else if (wwin->main_window != wwin->client_win) {
+        WApplication *wapp = wApplicationOf(wwin->main_window);
+
+	if (wapp) {
+            char *instance = wapp->main_window_desc->wm_instance;
+            char *class = wapp->main_window_desc->wm_class;
+            char *buffer;
+
+	    buffer = StrConcatDot(instance, class);
+	    key2 = WMCreatePLString(buffer);
+            wfree(buffer);
+
+            if (WMIsPropListEqualTo(key, key2)) {
+                WMMergePLDictionaries(winDic, appDic, True);
+                different |= different2;
+            } else {
+                WMRemoveFromPLDictionary(dict, key2);
+                if (different2) {
+                    WMPutInPLDictionary(dict, key2, appDic);
+                }
+            }
+            WMReleasePropList(key2);
+            WMReleasePropList(appDic);
+        }
+    } else {
+        WMMergePLDictionaries(winDic, appDic, True);
+        different |= different2;
+        WMReleasePropList(appDic);
+    }
 
     WMRemoveFromPLDictionary(dict, key);
     if (different) {
@@ -672,65 +713,7 @@ saveSettings(WMButton *button, InspectorPanel *panel)
     WMReleasePropList(key); 
     WMReleasePropList(winDic);
 
-    different = 0;
-    
-    /* application wide settings */
-    if (panel->inspected->main_window != panel->inspected->client_win
-	&& !(flags & UPDATE_DEFAULTS)) {
-	WApplication *wapp;
-	WMPropList *appDic;
-
-	wapp = wApplicationOf(panel->inspected->main_window);
-	if (wapp) {
-	    char *iconFile;
-		char *buffer;
-
-	    appDic = WMCreatePLDictionary(NULL, NULL, NULL);
-
-	    assert(wapp->main_window_desc->wm_instance!=NULL);
-	    assert(wapp->main_window_desc->wm_class!=NULL);
-
-		buffer = wmalloc(strlen(wapp->main_window_desc->wm_instance)
-						 +strlen(wapp->main_window_desc->wm_class)+4);
-	    strcat(strcpy(buffer, wapp->main_window_desc->wm_instance), ".");
-	    strcat(buffer, wapp->main_window_desc->wm_class);
-	    key = WMCreatePLString(buffer);
-            wfree(buffer);
-
-	    iconFile = wDefaultGetIconFile(wwin->screen_ptr, 
-					   wapp->main_window_desc->wm_instance,
-					   wapp->main_window_desc->wm_class,
-					   False);
-
-	    if (iconFile && iconFile[0]!=0) {
-		value = WMCreatePLString(iconFile);
-		different |= insertAttribute(dict, appDic, AIcon, value,
-					     flags&~IS_BOOLEAN);
-		WMReleasePropList(value);
-	    }
-		
-	    value = (WMGetButtonSelected(panel->appChk[0])!=0) ? Yes : No;
-	    different |= insertAttribute(dict, appDic, AStartHidden,  value, 
-					 flags);
-
-	    value = (WMGetButtonSelected(panel->appChk[1])!=0) ? Yes : No;
-	    different |= insertAttribute(dict, appDic, ANoAppIcon,  value, 
-					 flags);
-
-	    value = (WMGetButtonSelected(panel->appChk[2])!=0) ? Yes : No;
-	    different |= insertAttribute(dict, appDic, ACollapseAppIcons,  value, 
-					 flags);
-	    
-	    WMRemoveFromPLDictionary(dict, key);
-	    if (different) {
-		WMPutInPLDictionary(dict, key, appDic);
-	    }
-	    WMReleasePropList(key);
-	    WMReleasePropList(appDic);
-	}
-    }
-
-    WMWritePropListToFile(dict, db->path, True);
+    UpdateDomainFile(db);
 
     /* clean up */
     WMPLSetCaseSensitive(False);
@@ -827,8 +810,8 @@ applySettings(WMButton *button, InspectorPanel *panel)
     WSETUFLAG(wwin, no_close_button, WMGetButtonSelected(panel->attrChk[2]));
     WSETUFLAG(wwin, no_miniaturize_button, WMGetButtonSelected(panel->attrChk[3]));
     WSETUFLAG(wwin, no_border, WMGetButtonSelected(panel->attrChk[4]));
-    floating                   =  WMGetButtonSelected(panel->attrChk[5]);
-    sunken                     =  WMGetButtonSelected(panel->attrChk[6]);
+    floating = WMGetButtonSelected(panel->attrChk[5]);
+    sunken   = WMGetButtonSelected(panel->attrChk[6]);
     WSETUFLAG(wwin, omnipresent, WMGetButtonSelected(panel->attrChk[7]));
     WSETUFLAG(wwin, start_miniaturized, WMGetButtonSelected(panel->attrChk[8]));
     WSETUFLAG(wwin, start_maximized, WMGetButtonSelected(panel->attrChk[9]));
@@ -836,7 +819,7 @@ applySettings(WMButton *button, InspectorPanel *panel)
 
     WSETUFLAG(wwin, no_bind_keys, WMGetButtonSelected(panel->moreChk[0]));
     WSETUFLAG(wwin, no_bind_mouse, WMGetButtonSelected(panel->moreChk[1]));
-    skip_window_list           =  WMGetButtonSelected(panel->moreChk[2]);
+    skip_window_list = WMGetButtonSelected(panel->moreChk[2]);
     WSETUFLAG(wwin, no_focusable, WMGetButtonSelected(panel->moreChk[3]));
     WSETUFLAG(wwin, dont_move_off, WMGetButtonSelected(panel->moreChk[4]));
     WSETUFLAG(wwin, no_hide_others, WMGetButtonSelected(panel->moreChk[5]));
@@ -905,7 +888,7 @@ applySettings(WMButton *button, InspectorPanel *panel)
 	WSETUFLAG(wapp->main_window_desc, no_appicon,
 		  WMGetButtonSelected(panel->appChk[1]));
 
-	WSETUFLAG(wapp->main_window_desc, collapse_appicons,
+	WSETUFLAG(wapp->main_window_desc, shared_appicon,
 		  WMGetButtonSelected(panel->appChk[2]));
 	
         if (WFLAGP(wapp->main_window_desc, no_appicon))
@@ -1043,7 +1026,7 @@ revertSettings(WMButton *button, InspectorPanel *panel)
 		flag = WFLAGP(wapp->main_window_desc, no_appicon);
 		break;
 	     case 2:
-		flag = WFLAGP(wapp->main_window_desc, collapse_appicons);
+		flag = WFLAGP(wapp->main_window_desc, shared_appicon);
 		break;
 	    }
 	    WMSetButtonSelected(panel->appChk[i], flag);
@@ -1201,9 +1184,9 @@ createInspectorForWindow(WWindow *wwin, int xpos, int ypos,
 #endif
 
     spec_text = _("The configuration will apply to all\n"
-			"windows that have their WM_CLASS\n"
-			"property set to the above selected\n"
-			"name, when saved.");
+                  "windows that have their WM_CLASS\n"
+                  "property set to the above selected\n"
+                  "name, when saved.");
 
     panel = wmalloc(sizeof(InspectorPanel));
     memset(panel, 0, sizeof(InspectorPanel));
@@ -1612,10 +1595,10 @@ createInspectorForWindow(WWindow *wwin, int xpos, int ypos,
 			  "working correctly.");
 		break;
 	     case 2:
-		caption = _("Collapse application icons");
-		flag = WFLAGP(wapp->main_window_desc, collapse_appicons);
-		descr = _("Collapse application icons from other instances\n"
-			  "of this application into one.\n");
+		caption = _("Shared application icon");
+		flag = WFLAGP(wapp->main_window_desc, shared_appicon);
+                descr = _("Use a single shared application icon for all of\n"
+                          "the instances of this application.\n");
 		break;
 	    }
 	    panel->appChk[i] = WMCreateSwitchButton(panel->appFrm);
@@ -1650,7 +1633,7 @@ createInspectorForWindow(WWindow *wwin, int xpos, int ypos,
    
     /* if the window is a transient, don't let it have a miniaturize
      * button */
-    if (wWindowFor(wwin->transient_for)!=NULL)
+    if (wwin->transient_for!=None && wwin->transient_for!=scr->root_win)
 	WMSetButtonEnabled(panel->attrChk[3], False);
     else
 	WMSetButtonEnabled(panel->attrChk[3], True);
