@@ -2,7 +2,7 @@
  *
  *  WindowMaker window manager
  * 
- *  Copyright (c) 1997, 1998 Alfredo K. Kojima
+ *  Copyright (c) 1997-2002 Alfredo K. Kojima
  *  (cowardly remade from wdwrite.c; by judas@hell on Jan 26 2001)
  * 
  *  This program is free software; you can redistribute it and/or modify
@@ -125,7 +125,7 @@ int main(int argc, char **argv)
     if ((value = WMGetFromPLDictionary(dict, key)) == NULL)
 	return 2;	/* bad key */
 
-    printf("%s\n", WMGetFromPLString(value));
+    printf("%s\n", WMGetPropListDescription(value, True));
     return 0;
 }
 
